#ifndef MUI_CMANAGER_MCC_H
#define MUI_CMANAGER_MCC_H

#ifndef LIBRARIES_MUI_H
#include <libraries/mui.h>
#endif

/*
**  $VER: CManager_mcc.h 25.3 (18.1.2004)
**  Includes Release 25.3
**
**  CManager_mcc.h
**  Global bookmarks manager MUI class
**
**  Written by Simone Tellini and Alfonso Ranieri <alforan@tin.it>.
**
**  Released under the terms of the
**  GNU Public Licence version 2
*/

/***********************************************************************/
/*
** Tags
**
** These are Simone Tellini's tags:
** please, don't use them in other projects.
*/

#define TAGBASE  ((int)0xF76B0010)
#define CMTAG(n) (TAGBASE+(int)(n))

#define MUIC_CManager  "CManager.mcc"
#define CManagerObject MUI_NewObject(MUIC_CManager

/***********************************************************************/
/*
** CManager preferences
*/

/* Columns */
#define COLS_User   4
#define COLS_WWW    5
#define COLS_FTP    5
#define COLS_Chat   3
#define COLS_Telnet 2

/* List sorting */
struct SortOrder
{
    ULONG SortCol0;
    ULONG SortCol1;
    BYTE  Order[8];
    int   Width[8];
};

/* Entry list preferences */
struct ListPrefs
{
    ULONG            TreeW;
    ULONG            ListW;
    struct SortOrder Sorts[8];
    ULONG            Flags;
    ULONG            dummy[8];
};

/* ButtonsBar preferences */
struct ButtonsBarPrefs
{
    ULONG ViewMode;
    ULONG LabelPosition;
    ULONG BarPosition;
    ULONG Flags;
    ULONG dummy[8];
};

/* Flags */
enum
{
    SBFLG_Borderless = 1<<0,
    SBFLG_Raised     = 1<<1,
    SBFLG_Scaled     = 1<<2,
    SBFLG_Sunny      = 1<<3,
    SBFLG_Underscore = 1<<4,
    SBFLG_BarSpacer  = 1<<5,
    SBFLG_Hide       = 1<<6,
};

/* BarPosition */
enum
{
    SBPos_Horiz       = 1<<0,
    SBPos_TopMode     = 1<<1,
    SBPos_LeftMode    = 1<<2,
    SBPos_InnerMode   = 1<<3,

    SBPos_Top         = (1<<4)|SBPos_Horiz|SBPos_TopMode,
    SBPos_Bottom      = (1<<5)|SBPos_Horiz|SBPos_TopMode,
    SBPos_Left        = (1<<6)|SBPos_LeftMode,
    SBPos_Right       = (1<<7)|SBPos_LeftMode,
    SBPos_InnerTop    = (1<<8)|SBPos_Horiz|SBPos_InnerMode,
    SBPos_InnerBottom = (1<<9)|SBPos_Horiz|SBPos_InnerMode,
};

/* Visual quick preferences */
struct VisualPrefs
{
    struct ListPrefs       ListPrefs;
    struct ButtonsBarPrefs ButtonsBarPrefs;
};

/* Hotlists */
enum
{
    HL_IB,
    HL_V,
    HL_AWEB,
    HL_AMFTP,
    HL_MD2,
    HL_STFAX,
    HL_YAM,
    HL_AMIRC,
    HL_DOPUS,
    HL_DFA,
    HL_PINT,
    HL_GOODNEWS,
    HL_STRICQ,

    NUM_HOTLISTS
};

struct CMPrefs
{
    struct VisualPrefs              VisualPrefs;

    /* Settings */
    char                            Country[80];
    char                            CountryCode[9];
    ULONG                           Action;
    ULONG                           Options;

    /* Programs */
    UBYTE                           MailProg;
    char                            MailScript[256];
    char                            MailPath[256];

    UBYTE                           WWWProg;
    char                            WWWScript[256];
    char                            WWWPath[256];

    UBYTE                           FTPProg;
    char                            FTPScript[256];
    char                            FTPPath[256];
    char                            Local[256];
    ULONG                           FTPOptions;

    UBYTE                           IRCProg;
    char                            IRCScript[256];
    char                            IRCPath[256];

    UBYTE                           FaxProg;
    char                            FaxScript[256];
    char                            FaxPath[256];

    UBYTE                           TelnetProg;
    char                            TelnetScript[256];
    char                            TelnetPath[256];

    char                            SavedProg[256];

    /* Hotlists */
    char                            Hotlists[NUM_HOTLISTS][256];
    struct DateStamp                Stamps[NUM_HOTLISTS];

    /* Dialer */
    ULONG                           DialOptions;
    char                            DialProg[256];
    char                            SerDevice[256];
    UBYTE                           SerUnit;
    ULONG                           ModemType;
    char                            SerInit[256];
    char                            DialCommand[30];
    ULONG                           RedialDelay;
    ULONG                           RedialAttempts;
};

/* Action */
enum
{
    ACTION_EDIT,
    ACTION_SCRIPT,
    ACTION_SCRIPT_CLOSE,
    ACTION_SCRIPT_ICONIFY
};

/* Options */
enum
{
    OPT_SilentScan       = 1<<0,
    OPT_UseOpenURL       = 1<<1,
    OPT_AutoReload       = 1<<2,
    OPT_NtCheckChanges   = 1<<3,
    OPT_SaveOnExit       = 1<<4,
    OPT_SilentExit       = 1<<5,
    OPT_SaveOnChanges    = 1<<6,
    OPT_EditNew          = 1<<7,
    OPT_BoldSections     = 1<<8,
    OPT_BoldTitles       = 1<<9,
    OPT_NoListBar        = 1<<10,
    OPT_RightMouseActive = 1<<11,
    OPT_UseSavedProg     = 1<<12,
};

/* FTPOptions */
enum
{
    FTPOPT_UseLocal = 1<<0,
};

/* DialOptions */
enum
{
    DIALOPT_ExternalDialer = 1<<0,
    DIALOPT_SerShared      = 1<<1,
    DIALOPT_LogCalls       = 1<<2,
};

/***********************************************************************/
/*
** MCC
*/

#define MUIM_CManager_LoadData          CMTAG(1)
#define MUIM_CManager_SaveData          CMTAG(6)
#define MUIM_CManager_EditSelected      CMTAG(16)
#define MUIM_CManager_AddEntry          CMTAG(19)
#define MUIM_CManager_Import            CMTAG(20)
#define MUIM_CManager_Login             CMTAG(21)
#define MUIM_CManager_Export            CMTAG(22)
#define MUIM_CManager_Search            CMTAG(23)
#define MUIM_CManager_Cleanup           CMTAG(24)
#define MUIM_CManager_DoubleClick       CMTAG(28)
#define MUIM_CManager_PrefsWindow       CMTAG(30)
#define MUIM_CManager_LoginWindow       CMTAG(31)
#define MUIM_CManager_Reload            CMTAG(34)

/* Attributes */
#define MUIA_CManager_ListObject        CMTAG(0)  /* [..G.], Object *         */
#define MUIA_CManager_Changed           CMTAG(2)  /* [.SGN], BOOL             */
#define MUIA_CManager_Path              CMTAG(3)  /* [..G.], STRPTR           */
#define MUIA_CManager_NoMenu            CMTAG(4)  /* [ISGN], BOOL             */
#define MUIA_CManager_HideUsers         CMTAG(5)  /* [I...], BOOL             */
#define MUIA_CManager_HideWWW           CMTAG(6)  /* [I...], BOOL             */
#define MUIA_CManager_HideFTP           CMTAG(7)  /* [I...], BOOL             */
#define MUIA_CManager_HideChat          CMTAG(8)  /* [I...], BOOL             */
#define MUIA_CManager_CMData            CMTAG(9)  /* [..G.], struct CMData *  */
#define MUIA_CManager_AppDoubleClick    CMTAG(10) /* [IS..], struct Hook *    */
#define MUIA_CManager_HideTelnet        CMTAG(11) /* [I...], BOOL             */
#define MUIA_CManager_Prefs             CMTAG(17) /* [ISG.], struct CMPrefs * */
#define MUIA_CManager_AtHome            CMTAG(18) /* [I...], BOOL             */
#define MUIA_CManager_DoubleClickObject CMTAG(19) /* [IS..], Object *         */
#define MUIA_CManager_NoBar             CMTAG(20) /* [ISGN], BOOL             */
#define MUIA_CManager_NoGroups          CMTAG(21) /* [ISGN], BOOL             */
#define MUIA_CManager_ButtonsBar        CMTAG(25) /* [..G.], Object *         */

/* Structures */

struct MUIP_CManager_LoadData
{
    ULONG  MethodID;
    STRPTR User;
    STRPTR Path;
};

struct MUIP_CManager_SaveData
{
    ULONG  MethodID;
    STRPTR Path;
};

struct MUIP_CManager_AddEntry
{
    ULONG          MethodID;
    struct CMEntry *Entry;
    ULONG          Flags;
};

struct MUIP_CManager_Import
{
    ULONG  MethodID;
    ULONG  Type;
    STRPTR File;
    ULONG  Flags;
};
/* Import types */
enum
{
    MUIV_CManager_Import_Voyager,
    MUIV_CManager_Import_IB,
    MUIV_CManager_Import_AWeb,
    MUIV_CManager_Import_Users_CSV,
    MUIV_CManager_Import_AmFTP,
    MUIV_CManager_Import_THOR,
    MUIV_CManager_Import_MD,
    MUIV_CManager_Import_YAM,
    MUIV_CManager_Import_PINT,
    MUIV_CManager_Import_GoodNews,
    MUIV_CManager_Import_STRICQ,
    MUIV_CManager_Import_OldSTFax,
    MUIV_CManager_Import_STFax,
    MUIV_CManager_Import_AmIRC,
    MUIV_CManager_Import_DOpusFTP,
    MUIV_CManager_Import_DFA,
    MUIV_CManager_Import_Netscape,
    MUIV_CManager_Import_Users_CSVFullOE,
};

/* flags */
enum
{
    MUIV_CManager_Import_Filter = 1<<0, /* avoid duplicate entries */
};

struct MUIP_CManager_Export
{
    ULONG  MethodID;
    ULONG  Type;
    STRPTR File;
    ULONG  Flags;
};

/* Export types */
enum
{
    MUIV_CManager_Export_HTML_URLs,
    MUIV_CManager_Export_HTML_WWW,
    MUIV_CManager_Export_HTML_FTP,
    MUIV_CManager_Export_CSV_Users,

    MUIV_CManager_Export_YAM,
    MUIV_CManager_Export_AWeb,
    MUIV_CManager_Export_IBrowse,

    MUIV_CManager_Export_Netscape,
    MUIV_CManager_Export_CSVFullOE_Users,
};

/* Flags */
enum
{
    MUIV_CManager_Export_NoHead = 1<<0,
};


struct MUIP_CManager_Login
{
    ULONG  MethodID;
    STRPTR user;
};

struct MUIP_CManager_DoubleClick
{
    ULONG          MethodID;
    struct CMEntry *entry;
};

struct MUIP_CManager_PrefsWindow
{
    ULONG MethodID;
    ULONG page;
};

/* page */
enum
{
    MUIV_PrefsWindow_ActivePage_Options,
    MUIV_PrefsWindow_ActivePage_Locale,
    MUIV_PrefsWindow_ActivePage_ButtonsBar,
    MUIV_PrefsWindow_ActivePage_Programs,
    MUIV_PrefsWindow_ActivePage_DialerInternal,
    MUIV_PrefsWindow_ActivePage_DialerExternal,
    MUIV_PrefsWindow_ActivePage_Hotlists,
};


/***********************************************************************/

#endif /* MUI_CMANAGER_MCC_H */
