; StackAttack2 - Does the same as old stackattack, only nicer ;-)
;
; Written by Robin Cloutman
; This source code is Public Domain.
; Idea stolen from StackAttack by Georg Steger <steger@pass.dnet.it>.
;
; All patched functions may use a0/a1/d0 as scratch registers (before calling func)
;
; Ok, things that have happened -
;	2.0 - First new version, 100% asm, replacates old StackAttack.
;	2.1 - Changed the patch code to make it smaller/faster and more robust.
;	2.2 - Added IGNORE/K to ignore named tasks that should know what to set the stack.
;	      (Note, it checks the *new* prog, not the calling one!)
;	2.3 - Added a stackcheck function, so the patched function isn't called if there
;	      isn't enough free stack.
;	2.4 - Removed IGNORE/K as nobody used it ;-)
;	2.5 - Did the SystemTagList workaround from 1.3 (old version).
;	2.6 - No args gives default 8192 byte MINSTACK.
;	2.7 - Added SHOW/S to see settings. Changed MINSTACK and ADDSTACK so no longer /K
;	2.8 - Added NOTCLI/S, by default also sets the same as "Stack", extra checking for ADDSTACK/N
;	2.9 - Added AUTOSTACK/S, sets the stack depending on the amount of memory you have...
;
;    - Robin

	incdir	"include:"
	include	"exec/execbase.i"
	include	"exec/tasks.i"
	include	"exec/types.i"
	include	"exec/memory.i"
	include	"dos/dos.i"
	include	"dos/dosextens.i"
	include	"dos/dostags.i"
	include	"utility/tagitem.i"

	include	"lvo/exec_lib.i"
	include	"lvo/dos_lib.i"

VERSION	EQU	$0207						;Patch version, not program version
DEFSTACK	EQU	8192						;Default stack value
PORTNAME	MACRO
	dc.b	'StackAttack',0
 ENDM

 STRUCTURE SizeTag,0
	ULONG	st_StackSize	;NP_StackSize
	ULONG	st_Size			;memsize
	ULONG	st_TagMore		;TAG_MORE/TAG_DONE
	ULONG	st_More			;old taglist
 LABEL st_SIZEOF

tsta MACRO
	IFGE    __CPU-68020
	 tst.\0	\1
	ELSE
	 cmp.\0	#0,\1
	ENDC
 ENDM

	SECTION	MAIN,CODE
main:
	move.l	(4).w,a6
	suba.l	a3,a3
	lea.l		(_dos_name,pc),a1
	moveq		#37,d0
	jsr		(_LVOOpenLibrary,a6)
	movea.l	d0,a5
	beq.w		.nodos
	lea.l		(_template,pc),a0
	move.l	a0,d1
	lea.l		(_args,pc),a0
	move.l	a0,d2
	moveq		#0,d3
	movea.l	a5,a6
	jsr		(_LVOReadArgs,a6)
	movea.l	d0,a3
	tsta.l	a3
	beq.w		.noargs
.argsok
	move.l	(4).w,a6
	lea.l		(_stackattack,pc),a1
	jsr		(_LVOForbid,a6)						;*** Forbid ***
	jsr		(_LVOFindPort,a6)
	tst.l		d0
	beq.b		.noport
	movea.l	d0,a4
	lea.l		(version-_start,a4),a0
	cmp.w		#VERSION,(a0)
	beq.w		.portok
.noport
	move.l	#(_end-_start),d0
	moveq		#0,d1
	jsr		(_LVOAllocMem,a6)
	movea.l	d0,a4
	tsta.l	a4
	beq.w		.nomem									;No failure after here
	addq		#1,(LIB_OPENCNT,a5)					;*NOTE* increasing dos.library opencount so it doesn't close
	lea.l		_start,a0
	movea.l	a4,a1
	move.l	#(_end-_start),d0
	jsr		(_LVOCopyMemQuick,a6)
	lea.l		(stackattack-_start,a4),a0			;Add MessagePort
	lea.l		(lnName-_start,a4),a1
	move.l	a0,(a1)
	movea.l	a4,a1
	jsr		(_LVOAddPort,a6)
	lea.l		(newCreateProc-_start,a4),a0		;Patch CreateProc()
	move.l	a0,d0
	movea.w	#_LVOCreateProc,a0
	movea.l	a5,a1
	jsr		(_LVOSetFunction,a6)
	move.l	d0,(oldCreateProc-_start,a4)
	lea.l		(newCreateNewProc-_start,a4),a0	;Patch CreateNewProc()
	move.l	a0,d0
	movea.w	#_LVOCreateNewProc,a0
	movea.l	a5,a1
	jsr		(_LVOSetFunction,a6)
	move.l	d0,(oldCreateNewProc-_start,a4)
	lea.l		(newRunCommand-_start,a4),a0		;Patch RunCommand()
	move.l	a0,d0
	movea.w	#_LVORunCommand,a0
	movea.l	a5,a1
	jsr		(_LVOSetFunction,a6)
	move.l	d0,(oldRunCommand-_start,a4)
	lea.l		(newSystemTagList-_start,a4),a0	;Patch SystemTagList
	move.l	a0,d0
	movea.w	#_LVOSystemTagList,a0
	movea.l	a5,a1
	jsr		(_LVOSetFunction,a6)
	move.l	d0,(oldSystemTagList-_start,a4)
.portok
	movea.l	(_minStack,pc),a0						;Takes MINSTACK/N and applies to the patch
	tsta.l	a0
	beq.b		.nomin
	move.l	(a0),d0
	addq		#3,d0
	andi.b	#%11111100,d0							;Make sure it's a multiple of 4
	move.l	d0,(minStack-_start,a4)
.nomin
	movea.l	(_addStack,pc),a0						;Takes MINSTACK/N and applies to the patch
	tsta.l	a0
	beq.b		.noadd
	move.l	(a0),d0
	addq		#3,d0
	andi.b	#%11111100,d0							;Make sure it's a multiple of 4
	move.l	d0,(addStack-_start,a4)
.noadd
	movea.l	(_autoStack,pc),a0					;Do we set it automatically?
	tsta.l	a0
	beq.b		.notauto
	move.l	#0,(addStack-_start,a4)
	move.l	#MEMF_FAST|MEMF_TOTAL,d1			;Lets see how much fast memory there is...
	jsr		(_LVOAvailMem,a6)
	move.l	#8192,(minStack-_start,a4)			;Default 8k for anyone
	cmp.l		#16000000,d0
	blt.b		.notauto
	move.l	#16384,(minStack-_start,a4)		;16k for >16 megs
	cmp.l		#32000000,d0
	blt.b		.notauto
	move.l	#32768,(minStack-_start,a4)		;32k for >32 megs
	cmp.l		#64000000,d0
	blt.b		.notauto
	move.l	#65536,(minStack-_start,a4)		;64k for >64 megs
	cmp.l		#128000000,d0
	blt.b		.notauto
	move.l	#131072,(minStack-_start,a4)		;128k for >128 megs
.notauto
	jsr		(_LVOPermit,a6)						;*** Permit ***
	movea.l	(_notCLI,pc),a0						;Do we leave the CLI alone?
	tsta.l	a0
	bne.b		.nocli
	suba.l	a1,a1										;Now do "C:STACK xxxx"
	jsr		(_LVOFindTask,a6)						;Find this task/process...
	movea.l	d0,a0
	cmp.b		#NT_PROCESS,(LN_TYPE,a0)			;Are we a process?
	bne.b		.nocli
	move.l	(pr_CLI,a0),d0							;Find the CommandLineInterface
	lsl.l		#2,d0
	movea.l	d0,a0
	move.l	(minStack-_start,a4),d0
	lsr.l		#2,d0
	move.l	d0,(cli_DefaultStack,a0)			;Set the default CLI stack...
.nocli
	movea.l	(_showStack,pc),a0					;Do they want to SHOW/S it?
	tsta.l	a0
	beq.b		.exit
	lea.l		(_showme,pc),a0						;Got our display
	move.l	a0,d1
	lea.l		(minStack-_start,a4),a0				;Cheat and use the args themselves
	move.l	a0,d2
	movea.l	a5,a6
	jsr		(_LVOVPrintf,a6)						;And print it
.exit
	moveq		#0,d1
	moveq		#RETURN_OK,d0
	bra.b		.error
.noargs
	moveq		#ERROR_REQUIRED_ARG_MISSING,d1
	moveq		#RETURN_ERROR,d0
	bra.b		.error
.badignore
	moveq		#ERROR_BAD_TEMPLATE,d1
	moveq		#RETURN_ERROR,d0
	bra.b		.error
.nomem
	move.l	(4).w,a6
	jsr		(_LVOPermit,a6)						;*** Permit ***
	moveq		#ERROR_NO_FREE_STORE,d1
	moveq		#RETURN_FAIL,d0
	bra.b		.error
.nodos
	moveq		#RETURN_FAIL,d0
	rts

.error
	movea.l	a5,a6
	tsta.l	a3
	beq.b		.nofreeargs
	movem.l	d0-d1,-(sp)
	move.l	a3,d1
	jsr		(_LVOFreeArgs,a6)
	movem.l	(sp)+,d0-d1
.nofreeargs
	tst.l		d1
	beq.b		.noprintfault
	move.l	d0,-(sp)
	moveq		#0,d2
	jsr		(_LVOPrintFault,a6)
	move.l	(sp)+,d0
.noprintfault
	move.l	d0,-(sp)
	movea.l	a5,a1
	move.l	(4).w,a6
	jsr		(_LVOCloseLibrary,a6)
	move.l	(sp)+,d0
	rts

				dc.b	'$VER: StackAttack 2.9 (11.1.2004) 2000-2004 Robin Cloutman',0
_dos_name	dc.b	'dos.library',0
_template	dc.b	'MINSTACK=MIN/N,ADDSTACK=ADD/N,AUTOSTACK=AUTO/S,NOTCLI/S,SHOW/S',0
_showme		dc.b	'StackAttack MINSTACK=%ld ADDSTACK=%ld\n',0
_stackattack		PORTNAME
	CNOP	0,4
_args
_minStack	dc.l	0
_addStack	dc.l	0
_autoStack	dc.l	0
_notCLI		dc.l	0
_showStack	dc.l	0

	SECTION	PATCH,CODE
;************************** PATCH FROM HERE *****************************
_start
lnSucc				dc.l	0
lnPred				dc.l	0
lnType				dc.b	NT_MSGPORT
lnPri					dc.b	-128
lnName				dc.l	0							;PORTNAME
mpFlags				dc.b	PA_IGNORE
mpSigBit				dc.b	0
mpSigTask			dc.l	0
lhHead				dc.l	0
lhTail				dc.l	0
lhTailPred			dc.l	0
lhType				dc.b	0
lhPad					dc.b	0
version				dc.w	VERSION					;Also aligns to long again

minStack				dc.l	DEFSTACK
addStack				dc.l	0

oldCreateProc		dc.l	0
oldCreateNewProc	dc.l	0
oldRunCommand		dc.l	0
oldSystemTagList	dc.l	0
stackattack					PORTNAME

	CNOP	0,4
newCreateProc:											;CreateProc(name,pri,segList,stackSize)(d1/d2/d3/d4)
	moveq		#20,d0									;safe stack for this function
	jsr		(LowStackCheck,pc)					;sp -4+
	beq.b		.ignore
	move.l	d4,-(sp)									;sp -4
	cmp.l		(minStack,pc),d4
	bge.s		.ok
	move.l	(minStack,pc),d4
.ok
	add.l		(addStack,pc),d4
	addq		#3,d4
	andi.b	#%11111100,d4							;Make sure it's a multiple of 4
	jsr		(.ignore,pc)							;sp -4+
	move.l	(sp)+,d4									;sp 4+
	rts
.ignore
	movea.l	(oldCreateProc,pc),a0
	jmp		(a0)

	CNOP	0,4
newCreateNewProc:										;CreateNewProc(tags)(d1)
	moveq		#12+st_SIZEOF,d0						;safe stack for this function
	jsr		(LowStackCheck,pc)
	beq.b		.ignore
	movea.l	d1,a1
	move.l	a2,-(sp)									;sp -4
	suba.w	#st_SIZEOF,sp							;sp -st_SIZEOF
	movea.l	sp,a2
	jsr		(FindStack,pc)							;sp -8+
	move.l	a2,d1
	jsr		(.ignore,pc)							;sp -4+
	adda.w	#st_SIZEOF,sp							;sp st_SIZEOF+
	movea.l	(sp)+,a2									;sp 4+
	rts
.ignore
	movea.l	(oldCreateNewProc,pc),a0
	jmp		(a0)

	CNOP	0,4
newRunCommand:											;RunCommand(seg,stack,paramptr,paramlen)(d1/d2/d3/d4)
	moveq		#20,d0									;safe stack for this function
	jsr		(LowStackCheck,pc)					;sp -4+
	beq.b		.ignore
	move.l	d2,-(sp)									;sp -4
	cmp.l		(minStack,pc),d2
	bge.b		.ok
	move.l	(minStack,pc),d2
.ok
	add.l		(addStack,pc),d2
	addq		#3,d2
	andi.b	#%11111100,d2							;Make sure it's a multiple of 4
	jsr		(.ignore,pc)							;sp -4+
	move.l	(sp)+,d2									;sp 4+
	rts
.ignore
	movea.l	(oldRunCommand,pc),a0
	jmp		(a0)

	CNOP	0,4
newSystemTagList:										;SystemTagList(command,tags)(d1/d2)
	tst.l		d2											;see old stackattack1.3 docs
	beq.b		.ignore
	moveq		#12+st_SIZEOF,d0						;safe stack for this function
	jsr		(LowStackCheck,pc)					;sp -4+
	beq.b		.ignore
	move.l	a2,-(sp)									;sp -4
	suba.w	#st_SIZEOF,sp							;sp -st_SIZEOF (16)
	movea.l	sp,a2
	movea.l	d2,a1
	jsr		(FindStack,pc)							;sp -8+
	move.l	a2,d2
	jsr		(.ignore,pc)							;sp -4+
	move.l	(st_More,a2),d2
	adda.w	#st_SIZEOF,sp							;sp st_SIZEOF+ (16)
	movea.l	(sp)+,a2									;sp 4+
	rts
.ignore
	movea.l	(oldSystemTagList,pc),a0
	jmp		(a0)

;	SYNOPSIS
;   BOOL LowStackCheck( u_long Minimum );
;    d0                       d0
;	FUNCTION
;   Returns whether there is minimum (D0) bytes free on stack.
;   Also sets CC on result.
;	RESULT
;   FALSE - not enough stack
;   TRUE  - stack ok
;	NOTE
;   This function adds 200 to the value, if there's not enough
;   free stack to interrupt then we're in trouble...
;	STACK USAGE
;   4 bytes!!! (jsr only)
LowStackCheck:
	addi.l	#200,d0									;Minimum needed (d0-d7/a0-a7/fp?/cc)
	movea.l	(4).w,a0
	movea.l	(ThisTask,a0),a0
	cmp.l		(TC_SPLOWER,a0),sp
	ble.b		.ignore									;already stack overflow or bad StackSwap
	cmp.l		(TC_SPUPPER,a0),sp
	bge.b		.ignore									;already stack overflow or bad StackSwap
	add.l		(TC_SPLOWER,a0),d0
	cmp.l		d0,sp
	bge.b		.ignore
	moveq		#0,d0
.ignore
	tst.l		d0
	rts

FindStack:												;taglist in a1, memory in a2
	move.l	#NP_StackSize,d0
	jsr		(FindTag,pc)							;sp -4+
	cmp.l		(minStack,pc),d0
	bge.s		.stackok
	move.l	(minStack,pc),d0
.stackok
	add.l		(addStack,pc),d0
	addq		#3,d0
	andi.b	#%11111100,d0							;Make sure it's a multiple of 4
	move.l	#NP_StackSize,(a2)					;(st_StackSize,a2)
	move.l	d0,(st_Size,a2)
	tsta.l	a1
	beq.b		.notags
	move.l	#TAG_MORE,(st_TagMore,a2)
	move.l	a1,(st_More,a2)
	rts
.notags
	clr.l		(st_TagMore,a2)						;move.l #TAG_DONE,(st_TagMore,a2)
	rts

FindTag:													;taglist in a1, tag in d0, trash a0, return value (not tag) in d0
	tsta.l	a1
	beq.b		.notfound
	movea.l	a1,a0
.loop
	cmp.l		(a0),d0									;(ti_Tag,a0)
	beq.b		.found
	tst.l		(a0)										;cmp.l #TAG_DONE,(ti_Tag,a0)
	beq.b		.notfound
	cmp.l		#TAG_MORE,(a0)							;(ti_Tag,a0)
	beq.b		.more
	cmp.l		#TAG_SKIP,(a0)							;(ti_Tag,a0)
	beq.b		.skip
	addq		#ti_SIZEOF,a0
	bra.b		.loop
.notfound
	moveq		#0,d0
	rts
.found
	move.l	(ti_Data,a0),d0
	rts
.more
	movea.l	(ti_Data,a0),a0
	bra.b		.loop
.skip
	move.l	d0,-(sp)
	moveq		#ti_SIZEOF,d0
	mulu		(ti_Data,a0),d0
	addq		#ti_SIZEOF,d0
	adda.l	d0,a0
	move.l	(sp)+,d0
	bra.b		.loop

	CNOP	0,4
_end

;************************** PATCH TO HERE *****************************
