<?php
/* vim: set expandtab tabstop=4 shiftwidth=4: */

if(!defined("ZUUL_HOME_DIR"))
    define("ZUUL_HOME_DIR", "./");

// Require File Class
//require_once(ZUUL_HOME_DIR . "/class/Debug.php");
//require_once(ZUUL_HOME_DIR . "/class/Message.php");
require_once(ZUUL_HOME_DIR . "/class/Translator.php");
require_once(ZUUL_HOME_DIR . "/class/File.php");
require_once(ZUUL_HOME_DIR . "/class/Server.php");
require_once(ZUUL_HOME_DIR . "/class/Search.php");
require_once(ZUUL_HOME_DIR . "/class/SearchResult.php");

/**
* Core class
* 
* Core engine object.  The MCP, if you will that interfaces to mldonkey
*
* @version  $Id: Core.php,v 1.113 2003/08/19 20:05:47 cyb0rg Exp $
* @author   Don Seiler <don@NOSPAM.seiler.us>
* @package  zuul
*/
class ZUUL_Core {

    /**
    * The version of Zuul running
    * @var      string
    * @access   public
    */
    var $version = "1.1.2";


    /**
    * Array containing settings for Zuul
    * @var      array
    * @access   public
    */
    var $settings = array();


    /**
    * Constructor for ZUUL_Core class
    *
    * Loads all settings and initializes what is needed.
    *
    * @author   Don Seiler <don@NOSPAM.seiler.us>
    * @access   public
    */
    function ZUUL_Core()
    {
        if (!$this->loadSettings())
            exit("ZUUL_Core Initialization ERROR!  Config file not found!");        
        $missingsettings = $this->checkSettings();
        if (count($missingsettings) > 0) {
            $errorstring = "ZUUL_Core Initialization ERROR!<p /><p />\nConfig file incomplete.  The following values are missing:<br />\n<ul>\n";
            foreach ($missingsettings as $setting) {
                $errorstring .= "<li>$setting</li>\n";
            }
            $errorstring .= "</ul>\n<p />\n";
            $errorstring .= "If you have recently upgraded, please check config.php.sample for any new settings.\n";
            exit($errorstring);

        }

        $this->initSession();
        $_SESSION["translator"] = new ZUUL_Translator($this->settings["language"]);
        $GLOBALS["translator"] = &$_SESSION["translator"];
    }// END FUNC ZUUL_Core


    /**
    * Initializes and names session object
    *
    * @author   Don Seiler <don@NOSPAM.seiler.us>
    * @access   public
    */
    function initSession()
    {
        $sessionName = md5($this->settings["home_http"]);
        session_name($sessionName);
        session_start($sessionName);
    }// END FUNC initSession()


    /**
    * Destroys sessions
    *
    * @author   Don Seiler <don@NOSPAM.seiler.us>
    * @access   public
    */
    function killSession()
    {
        $_SESSION = array();
        unset($_SESSION);
        session_destroy();
    }// END FUNC killSession()


    /**
    * Loads core settings into $this->settings array
    * If you change this function, change checkSettings() too.
    *
    * @author   Don Seiler <don@NOSPAM.seiler.us>
    */
    function loadSettings()
    {
        if (file_exists(ZUUL_HOME_DIR . "/conf/config.php")) {
            require(ZUUL_HOME_DIR . "/conf/config.php");
            $this->settings["home_dir"]         = $home_dir;
            $this->settings["home_http"]        = $home_http;
            $this->settings["language"]         = $language;
            $this->settings["theme"]            = $theme;
            $this->settings["mldonkey_startup"] = $mldonkey_startup;
            $this->settings["mldonkey_host"]    = $mldonkey_host;
            $this->settings["mldonkey_port"]    = $mldonkey_port;
            $this->settings["mldonkey_auth"]    = $mldonkey_auth;
            $this->settings["mldonkey_user"]    = $mldonkey_user;
            $this->settings["mldonkey_passwd"]  = $mldonkey_passwd;
            $this->settings["default_max_name_len"] = $default_max_name_len;
            $this->settings["use_javascript"]   = $use_javascript;
            define("ZUUL_HOME_HTTP", $this->settings["home_http"]);
            require(ZUUL_HOME_DIR . "/themes/" . $this->settings["theme"] . "/template.php");
            return TRUE;
        } else {
            return FALSE;
        }
    }// END FUNC loadSettings


    /**
    * Checks settings for completeness
    *
    * @author   CJ Kucera <zuul@NOSPAM.apocalyptech.com>
    */
    function checkSettings()
    {
        $retarray = array();
        $setarray = array("home_dir",
            "home_http",
            "language",
            "theme",
            "mldonkey_startup",
            "mldonkey_host",
            "mldonkey_port",
            "default_max_name_len",
            "use_javascript");
        foreach ($setarray as $setting) {
            if (is_null($this->settings[$setting]) or $this->settings[$setting] === "") {
                $retarray[] = $setting;
            }
        }
        return $retarray;
    }// END FUNC checkSettings


    /**
    * Makes calls to mldonkey telnet service
    * Pass in any number of commands; will return an array of
    * lines corresponding to the command if one command is passed
    * in, or an array of array of lines if more than one command
    * was passed, corresponding to each command.  Each passed-in
    * command can be an array if desired, and each element of the
    * array will be treated as a command to run.
    *
    * @author   Don Seiler <don@NOSPAM.seiler.us>
    * @author   CJ Kucera <zuul@NOSPAM.apocalyptech.com>
    */
    function mldonkey()
    {
        $retresults = array();
        $commands = array();
        $foundarray = false;

        if($this->alive()) {
            $fp = fsockopen($this->settings["mldonkey_host"], $this->settings["mldonkey_port"]);

            if($this->settings["mldonkey_auth"]) {
                $authstring = "auth ";
                if(!empty($this->settings["mldonkey_user"])) {
                    $authstring .= $this->settings["mldonkey_user"] . " ";
                }
                $authstring .= " " . $this->settings["mldonkey_passwd"];
    
                $commands = array($authstring);
            }

            // Force term_ansi to true
            $commands[] = "set term_ansi true";

            foreach(func_get_args() as $command) {
                if (is_array($command)) {
                    $commands = array_merge($commands, $command);
                    $foundarray = true;
                } else {
                    $commands[] = $command;
                }
            }

            $commands[] = "q";

            $commandstring = implode("\n", $commands);
            fputs($fp, "$commandstring\n");
        
            $i = 0;
            $j = 0;
            $results = array();
            while (!feof($fp)) {
                $results[] = fgets($fp, 4096);
                // Parse code lifted from phpeselgui
                // http://phpeselgui.sourceforge.net
                $results[$i] = preg_replace("/\[\dm/", "", $results[$i]);
                $results[$i] = preg_replace("/\[3\dm/", "", $results[$i]);
                $results[$i] = preg_replace("/\[2;37;0m/", "", $results[$i]);
                //if (!$conf->showPasswords) {
                    //$results[$i] = preg_replace("/auth\ .*$/", "*** phpEselGui: Password output hidden", $results[$i]);
                //}
                $results[$i] = preg_replace("/^\s*/", "", $results[$i]);
                $results[$i] = preg_replace("/\e/", "", $results[$i]);
                $results[$i] = preg_replace("/.*/", "", $results[$i]);
                if (trim($results[$i]) == "> > " . $commands[$j]) {
                    reset($results);
                    $retresults[] = $results;
                    $results = array();
                    $i = 0;
                    $j++;
                } else {
                    $i++;
                }
            }
            $retresults[] = $results;

            // Shift the Welcome message out
            array_shift($retresults);

            // Check for authorization
            if (preg_match("/^Command not authorized/", $retresults[0][0])) {
                echo "<span class=\"error\">";
                echo $GLOBALS["translator"]->translate("Error authenticating to MLDonkey.  Check your authentication settings in config.php");
                echo "</span>\n";
                return NULL;
            } elseif (preg_match("/^Bad login\/password/", $retresults[0][0])) {
                echo "<span class=\"error\">";
                echo $GLOBALS["translator"]->translate("Bad username or password specified in config.php");
                echo "</span>\n";
                return NULL;
            }

            // Shift authorization out
            if($this->settings["mldonkey_auth"]) {
                array_shift($retresults);
            }

            // Shift our term_ansi check out
            array_shift($retresults);

            // Check to see if term_ansi was false when we ran the commands
            if (count($retresults) == 0) {
                echo "<span class=\"error\">";
                echo $GLOBALS["translator"]->translate("No valid output was received from MLDonkey.  This is usually because of the variable term_ansi being set to false.  Zuul has set term_ansi to true, so the next page load should be fine.");
                echo "</span>\n";
                return NULL;
            }

            // Now return everything
            if (func_num_args() > 1 or $foundarray) {
                return $retresults;
            } else {
                return $retresults[0];
            }

        } else {
            return NULL;
        }

    }// END FUNC mldonkey


    /**
    * Lists files being uploaded and the uploaders
    *
    * @author   Don Seiler <don@NOSPAM.seiler.us>
    */
    function _uploaders()
    {
        if(isset($_REQUEST["ZUUL_SORTKEY"]))
            $sortkey = $_REQUEST["ZUUL_SORTKEY"];
        else
            $sortkey = "score";

        if(isset($_REQUEST["ZUUL_SORTORDER"]))
            $sortorder = $_REQUEST["ZUUL_SORTORDER"];
        else
            $sortorder = "DESC";

        $headers = array();
        $headers["nr"]          = $GLOBALS["translator"]->translate("No.");
        $headers["name"]        = $GLOBALS["translator"]->translate("Name");
        $headers["user"]        = $GLOBALS["translator"]->translate("User");
        $headers["client"]      = $GLOBALS["translator"]->translate("Client");
        $headers["score"]       = $GLOBALS["translator"]->translate("Score");
        $headers["uploaded"]    = $GLOBALS["translator"]->translate("Uploaded");
        $headers["ip"]          = $GLOBALS["translator"]->translate("IP Address");
        $headers["network"]     = $GLOBALS["translator"]->translate("Network");

        $content = "<div align=\"center\">\n<table border=\"0\">\n";
        $content .= "<tr class=\"bg_dark\">\n";
        foreach($headers as $key=>$value)
            if($key != "action")
                $content .= "<th><a class=\"header\" href=\"index.php?ZUUL_CORE_OP=uploaders&ZUUL_SORTKEY=$key&ZUUL_SORTORDER=ASC\"></a>$value<a class=\"header\" href=\"index.php?ZUUL_CORE_OP=uploaders&ZUUL_SORTKEY=$key&ZUUL_SORTORDER=DESC\"></a></th>\n";
            else
                $content .= "<th>$value</th>\n";
        $content .= "</tr>\n";

        $mldarray = $this->mldonkey("uploaders");

        $ufiles = array();
        $files = array();
        $file = NULL;
        foreach($mldarray as $line) {
            if(preg_match("/^\[|^filename:/", $line)) {
                if(preg_match("/^\[/", $line)) {
                    $file = new ZUUL_File($line,TRUE);
                } else {
                    $file->setFilename(substr($line,10));
                    $ufiles[] = $file;
                    $file = NULL;
                }
            }
        }

        foreach($ufiles as $file) {

            if(isset($file)) {
                $key = NULL;
                switch($sortkey) {
                    case "network":
                        $key = $file->network;
                        break;

                    case "nr":
                        $key = $file->nr;
                        break;

                    case "name":
                        $key = $file->name;
                        break;

                    case "user":
                        $key = $file->user;
                        break;

                    case "client":
                        $key = $file->client;
                        break;

                    case "score":
                        $key = $file->uploadscore;
                        break;

                    case "ip":
                        $key = $file->ip;
                        break;

                    case "uploaded":
                        $key = $file->uploaded;
                        break;

                    default:
                        $key = $file->uploaded;
                }
                $files[$key][] = $file;
            }
        }

        // Sort
        if($sortorder == "DESC")
            krsort($files);
        else
            ksort($files);
        reset($files);

        $c = 0;
        foreach($files as $filelevel) {
            foreach($filelevel as $file) {
            // Display
            $content .= "<tr class=\"";
            if($c%2 == 0) $content .= "list";
            else $content .= "alt_list";
            $content .= "\">\n";
            $content .= "<td>$file->nr</td>\n";
            $content .= "<td>$file->name</td>\n";
            $content .= "<td>$file->user</td>\n";
            $content .= "<td>$file->client</td>\n";
            $content .= "<td>$file->uploadscore</td>\n";
            $content .= "<td>" . $file->hrUploaded() . "</td>\n";
            $content .= "<td>$file->ip</td>\n";
            $content .= "<td>$file->network</td>\n";
            $content .= "</tr>\n";
            $c++;
            }
        }

        $content .= "</table>\n</div>\n";
        return $content;
    }// END FUNC _uploaders


    /**
    * Lists files being downloaded
    *
    * @author   Don Seiler <don@NOSPAM.seiler.us>
    */
    function _vd()
    {
        if(isset($_REQUEST["nr"]) && $_REQUEST["ZUUL_CORE_OP"] == "vd")
            return $this->_vddetails();

        $content = NULL;

        if(isset($_REQUEST["ZUUL_SORTKEY"]))
            $sortkey = $_REQUEST["ZUUL_SORTKEY"];
        else
            $sortkey = "speed";

        if(isset($_REQUEST["ZUUL_SORTORDER"]))
            $sortorder = $_REQUEST["ZUUL_SORTORDER"];
        else
            $sortorder = "DESC";

        $mldarray = $this->mldonkey("vd");

        $files = array();
        $finishedfiles = array();
        $finished = FALSE;
        foreach($mldarray as $line) {
            if (preg_match("/^Downloaded\s{1}[^0]{1}\d*\s{1}/", $line)) {
                // Begin of finished sections
                $finished = TRUE;
                continue;
            }

            $file = new ZUUL_File($line, FALSE, $finished);
            if(isset($file)) {
                $key = NULL;
                switch($sortkey) {
                    case "network":
                        $key = $file->network;
                        break;

                    case "nr":
                        $key = $file->nr;
                        break;

                    case "eta":
                        $key = $file->eta;
                        break;

                    case "name":
                        $key = $file->name;
                        break;

                    case "percent":
                        $key = $file->percent;
                        break;

                    case "priority":
                        $key = $file->priority;
                        break;

                    case "size":
                        $key = $file->size;
                        break;

                    case "downloaded":
                        $key = $file->downloaded;
                        break;

                    case "old":
                        $key = $file->old;
                        break;

                    case "md4hash":
                        $key = $file->md4hash;
                        break;

                    default:
                        $key = $file->speed;
                }
                $files[$key][] = $file;
            }

            // Sort
            if($sortorder == "DESC")
                krsort($files);
            else
                ksort($files);
            reset($files);
        }

        $content .= $this->_checkboxJavascript();

        // Make our list of files to process
        $running_files = array();
        $paused_files = array();
        foreach($files as $filelevel) {
            foreach($filelevel as $file) {
                if($file->finished) {
                    continue;
                } elseif ($file->isPaused()) {
                    $paused_files[] = $file;
                    continue;
                } else {
                    $running_files[] = $file;
                }
            }
        }

        // Show running files
        if (count($running_files) > 0) {
            $headers = array();
            $headers["action"]      = "";
            $headers["nr"]          = $GLOBALS["translator"]->translate("No.");
            $headers["name"]        = $GLOBALS["translator"]->translate("Name");
            $headers["percent"]     = $GLOBALS["translator"]->translate("%");
            $headers["downloaded"]  = $GLOBALS["translator"]->translate("Downloaded");
            $headers["size"]        = $GLOBALS["translator"]->translate("Size");
            $headers["old"]         = $GLOBALS["translator"]->translate("Old");
            $headers["priority"]    = $GLOBALS["translator"]->translate("Priority");
            $headers["speed"]       = $GLOBALS["translator"]->translate("Speed");
            $headers["eta"]         = $GLOBALS["translator"]->translate("ETA");
            $headers["network"]     = $GLOBALS["translator"]->translate("Network");

            $content .= "<form name=\"vd\" action=\"index.php\" method=\"post\">\n";
            $content .= "<h2>" . $GLOBALS["translator"]->translate("Current Downloads") . "</h2><br>\n";
            $content .= "<table cellspacing=\"0\" cellpadding=\"2\" border=\"0\" width=\"100%\">\n";
            //$content .= "<tr><td colspan=\"11\">\n";
            //$content .= $this->_checkboxLinks("vd");
            //$content .= "</td></tr>\n";
            $content .= "<tr class=\"bg_dark\" bgcolor=\"#909090\">\n";
            foreach($headers as $key=>$value)
                if($key != "action")
                    $content .= "<td><font size=2><a class=\"header\" href=\"index.php?ZUUL_CORE_OP=vd&ZUUL_SORTKEY=$key&ZUUL_SORTORDER=ASC\"><img src=\"".ZUUL_HOME_HTTP."themes/default/images/down.png\" border=0 width=7 height=9 vspace=0 hspace=0 alt=\"sort\"></a><b>$value</b><a class=\"header\" href=\"index.php?ZUUL_CORE_OP=vd&ZUUL_SORTKEY=$key&ZUUL_SORTORDER=DESC\"><img src=\"".ZUUL_HOME_HTTP."themes/default/images/up.png\" border=0 width=7 height=9 vspace=0 hspace=0 alt=\"sort\"></a></font></td>\n";
                else
                    $content .= "<td><b><font size=2>$value</font></b></td>\n";
            $content .= "</tr>\n";

            $c = 0;

            $total_downloaded = 0;
            $total_size = 0;
            $bgtoggler = 0;
            foreach($running_files as $file) {
                $tdbgcolor = ($bgtoggler) ? "#979797" : "#a5a5a5";
                $total_downloaded += $file->downloaded;
                $total_size += $file->size;
                $content .= "<tr class=\"";
                if($c%2 == 0) $content .= "list";
                else $content .= "alt_list";
                $content .= "\">\n";
                $content .= "<td rowspan=\"2\" bgcolor=\"$tdbgcolor\">\n";
                $content .= "<input type=\"checkbox\" name=\"nr[]\" value=\"$file->nr\" />\n";
                $content .= "<input type=\"hidden\" name=\"name_" . $file->nr . "\" value=\"$file->name\" />\n";
                $content .= "</td>\n";
                $content .= "<td rowspan=\"2\" bgcolor=\"$tdbgcolor\">$file->nr</td>\n";
                $content .= "<td bgcolor=\"$tdbgcolor\"><font size=2><b><a href=\"index.php?ZUUL_CORE_OP=vd&nr=$file->nr\">$file->name</a></b></font></td>\n";
                $content .= "<td bgcolor=\"$tdbgcolor\" rowspan=\"2\">$file->percent</td>\n";
                $content .= "<td bgcolor=\"$tdbgcolor\" rowspan=\"2\">" . $file->hrDownloaded() . "</td>\n";
                $content .= "<td bgcolor=\"$tdbgcolor\" rowspan=\"2\">" . $file->hrSize() . "</td>\n";
                $content .= "<td bgcolor=\"$tdbgcolor\" rowspan=\"2\">$file->old</td>\n";
                $content .= "<td bgcolor=\"$tdbgcolor\" rowspan=\"2\">" . $file->hrPriority() . "</td>\n";
                if ($file->speed != "-") {
                    if ($file->speed == -1)
                        $class="speed_paused";
                    elseif ($file->speed <= 1)
                        $class="speed_slowest";
                    elseif ($file->speed <= 3)
                        $class="speed_slow";
                    elseif ($file->speed <= 5)
                        $class="speed_medium";
                    elseif ($file->speed <= 10)
                        $class="speed_fast";
                    elseif ($file->speed <= 20)
                        $class="speed_faster";
                    elseif ($file->speed > 20)
                        $class="speed_fastest";
                    $content .= "<td  bgcolor=\"$tdbgcolor\" rowspan=\"2\" class=\"$class\">\n";
                } else {
                    $content .= "<td  bgcolor=\"$tdbgcolor\" rowspan=\"2\">\n";
                }
                if($file->speed >= 0) { $content .= "$file->speed\n"; }
                else { $content .= $GLOBALS["translator"]->translate("Paused") . "\n"; }
                $content .= "</td>\n";
                $content .= "<td  bgcolor=\"$tdbgcolor\" rowspan=\"2\">" . $file->hrETA() . "</td>\n";
                $content .= "<td  bgcolor=\"$tdbgcolor\" rowspan=\"2\">$file->network</td>\n";
                $content .= "</tr>\n";
                $content .= "<tr class=\"";
                if($c%2 == 0) $content .= "list";
                else $content .= "alt_list";
                $content .= "\">\n";
                $pppp = sprintf("%02.0f",$file->percent); 
                $content .= "<td  bgcolor=\"$tdbgcolor\" valign=\"center\"><font size=1><img src=\"themes/" . $this->settings["theme"] . "/images/progressbar.png\" width=\"".( $pppp * 2)."\" height=\"3\"><img src=\"themes/" . $this->settings["theme"] . "/images/progressbar2.png\" width=\"".((100 - $pppp) * 2)."\" height=\"3\"></font></td>\n";
                $content .= "</tr>\n";
                $c++;
                $bgtoggler = 1 - $bgtoggler;
            }

            $content .= "<tr><td colspan=\"11\">\n";
            $content .= $this->_checkboxLinks("vd");
            $content .= "</td></tr>\n";
            $content .= "<tr><td colspan=\"3\">\n";
            $content .= "<input type=\"submit\" name=\"action_pause\" value=\"" . $GLOBALS["translator"]->translate("Pause") . "\" /> \n";
            $content .= "<input type=\"submit\" name=\"action_cancel\" value=\"" . $GLOBALS["translator"]->translate("Cancel") . "\" /> \n";
            $content .= "<input type=\"submit\" name=\"render_ed2k\" value=\"" . $GLOBALS["translator"]->translate("Show ED2K Links") . "\" /> \n";
            $content .= "</td>\n";
            $content .= "<td colspan=\"8\" rowspan=\"2\" align=\"right\" valign=\"top\">\n";
            $content .= "<strong>" . ZUUL_File::hrSizes($total_downloaded) . " " . $GLOBALS["translator"]->translate("of") . " " . ZUUL_File::hrSizes($total_size) . " " . $GLOBALS["translator"]->translate("downloaded") . "</strong>\n";
            $content .= "</td>\n";
            $content .= "</tr>\n";
            $content .= "<tr><td colspan=\"3\">\n";
            $content .= "<select name=\"priority\" size=\"1\">\n";
            $priorities = ZUUL_File::getPriorities();
            foreach($priorities as $key=>$value) {
                $content .= "   <option value=\"$key\">$value</option>\n";
            }
            $content .= "</select>\n";
            $content .= "<input type=\"submit\" name=\"setpriority\" value=\"" . $GLOBALS["translator"]->translate("Set Priority") . "\" />\n";
            $content .= "</td></tr>\n";
            $content .= "</table></div>\n";
            $content .= "<input type=\"hidden\" name=\"ZUUL_CORE_OP\" value=\"bulk\" />\n";
            $content .= "</form>\n";

            $content .= "<br />\n<br />\n";

        }

        // Show paused files
        if (count($paused_files) > 0) {
            $headers = array();
            $headers["action"]      = "";
            $headers["nr"]          = $GLOBALS["translator"]->translate("No.");
            $headers["name"]        = $GLOBALS["translator"]->translate("Name");
            $headers["percent"]     = $GLOBALS["translator"]->translate("%");
            $headers["downloaded"]  = $GLOBALS["translator"]->translate("Downloaded");
            $headers["size"]        = $GLOBALS["translator"]->translate("Size");
            $headers["priority"]       = $GLOBALS["translator"]->translate("Priority");
            $headers["network"]     = $GLOBALS["translator"]->translate("Network");

            $content .= "<form name=\"vdp\" action=\"index.php\" method=\"post\">\n";
            $content .= "<div align=\"center\">\n";
            $content .= "<h3>" . $GLOBALS["translator"]->translate("Paused Downloads") . "</h3>\n";
            $content .= "<table cellspacing=\"0\" cellpadding=\"2\" border=\"0\" width=\"100%\">\n";
            $content .= "<tr><td colspan=\"11\">\n";
            $content .= $this->_checkboxLinks("vdp");
            $content .= "</td></tr>\n";
            $content .= "<tr class=\"bg_dark\">\n";
            foreach($headers as $key=>$value)
                if($key != "action")
                    $content .= "<th><a class=\"header\" href=\"index.php?ZUUL_CORE_OP=vd&ZUUL_SORTKEY=$key&ZUUL_SORTORDER=ASC\"></a>$value<a class=\"header\" href=\"index.php?ZUUL_CORE_OP=vd&ZUUL_SORTKEY=$key&ZUUL_SORTORDER=DESC\"></a></th>\n";
                else
                    $content .= "<th>$value</th>\n";
            $content .= "</tr>\n";

            $c = 0;
            $total_downloaded = 0;
            $total_size = 0;
            foreach($paused_files as $file) {
                if($file->finished or !$file->isPaused())
                    continue;
                $total_downloaded += $file->downloaded;
                $total_size += $file->size;
                $content .= "<tr class=\"";
                if($c%2 == 0) $content .= "list";
                else $content .= "alt_list";
                $content .= "\">\n";
                $content .= "<td rowspan=\"2\">\n";
                $content .= "<input type=\"checkbox\" name=\"nr[]\" value=\"$file->nr\" />\n";
                $content .= "<input type=\"hidden\" name=\"name_" . $file->nr . "\" value=\"$file->name\" />\n";
                $content .= "</td>\n";
                $content .= "<td rowspan=\"2\">$file->nr</td>\n";
                $content .= "<td><a href=\"index.php?ZUUL_CORE_OP=vd&nr=$file->nr\">$file->name</a></td>\n";
                $content .= "<td rowspan=\"2\">$file->percent</td>\n";
                $content .= "<td rowspan=\"2\">" . $file->hrDownloaded() . "</td>\n";
                $content .= "<td rowspan=\"2\">" . $file->hrSize() . "</td>\n";
                $content .= "<td rowspan=\"2\">" . $file->hrPriority() . "</td>\n";
                $content .= "<td rowspan=\"2\">$file->network</td>\n";
                $content .= "</tr>\n";
                $content .= "<tr class=\"";
                if($c%2 == 0) $content .= "list";
                else $content .= "alt_list";
                $content .= "\">\n";
                $content .= "<td><img src=\"themes/" . $this->settings["theme"] . "/images/progressbar.png\" width=\"$file->percent%\" height=\"4\"></td>\n";
                $content .= "</tr>\n";
                $c++;
            }

            $content .= "<tr><td colspan=\"11\">\n";
            $content .= $this->_checkboxLinks("vdp");
            $content .= "</td></tr>\n";
            $content .= "<tr><td colspan=\"3\">\n";
            $content .= "<input type=\"submit\" name=\"action_resume\" value=\"" . $GLOBALS["translator"]->translate("Resume") . "\" /> \n";
            $content .= "<input type=\"submit\" name=\"action_cancel\" value=\"" . $GLOBALS["translator"]->translate("Cancel") . "\" /> \n";
            $content .= "<input type=\"submit\" name=\"render_ed2k\" value=\"" . $GLOBALS["translator"]->translate("Show ED2K Links") . "\" /> \n";
            $content .= "</td>\n";
            $content .= "<td colspan=\"8\" rowspan=\"2\" align=\"right\" valign=\"top\">\n";
            $content .= "<strong>" . ZUUL_File::hrSizes($total_downloaded) . " " . $GLOBALS["translator"]->translate("of") . " " . ZUUL_File::hrSizes($total_size) . " " . $GLOBALS["translator"]->translate("downloaded") . "</strong>\n";
            $content .= "</td>\n";
            $content .= "</tr>\n";
            $content .= "<tr><td colspan=\"3\">\n";
            $content .= "<select name=\"priority\" size=\"1\">\n";
            $priorities = ZUUL_File::getPriorities();
            foreach($priorities as $key=>$value) {
                $content .= "   <option value=\"$key\">$value</option>\n";
            }
            $content .= "</select>\n";
            $content .= "<input type=\"submit\" name=\"setpriority\" value=\"" . $GLOBALS["translator"]->translate("Set Priority") . "\" />\n";
            $content .= "</td></tr>\n";
            $content .= "</table></div>\n";
            $content .= "<input type=\"hidden\" name=\"ZUUL_CORE_OP\" value=\"bulk\" />\n";
            $content .= "</form>\n";

            $content .= "<br />\n<br />\n";
        }

        return $content;

    }// END FUNC _vd


    /**
    * Shows file download details
    *
    * @author   Don Seiler <don@NOSPAM.seiler.us>
    */
    function _vddetails()
    {
        $content = NULL;
        $file = NULL;
        $resultsarray = $this->mldonkey("set max_name_len 200", "vd", "set max_name_len " . $this->settings["default_max_name_len"], "vd " . $_REQUEST["nr"]);
        $mldarray = $resultsarray[1];
        foreach($mldarray as $line) {
            $file = new ZUUL_File($line);
            if($file->nr == $_REQUEST["nr"])
                break;
        }
        $file->getDetails($resultsarray[3]);

        $numsources = sizeof($file->sources);

        $content .= "<h2>$file->name</h2>\n";

        $content .= "<div align=\"center\">\n";
        $content .= "<table border=\"1\" cellpadding=\"5\" cellspacing=\"0\">\n<tr>\n";

        // Print form to change priority
        $priorities = $file->getPriorities();
        $content .= "<td>\n<form method=\"post\" action=\"index.php\" name=\"setpriority\">\n";
        $content .= "<input type=\"hidden\" name=\"ZUUL_CORE_OP\" value=\"priority\" />\n";
        $content .= "<input type=\"hidden\" name=\"nr\" value=\"$file->nr\" />\n";
        $content .= "<select name=\"priority\" size=\"1\">\n";
        foreach($priorities as $key=>$value) {
            $content .= "   <option value=\"$key\"";
            if($key == $file->priority) { $content .= " selected"; }
            $content .= ">$value</option>\n";
        }
        $content .= "</select>\n";
        $content .= "<input type=\"submit\" name=\"setpriority\" value=\"" . $GLOBALS["translator"]->translate("Set Priority") . "\" />\n";
        $content .= "</form>\n</td>\n";
        $content .= "<td>\n<form method=\"post\" action=\"index.php\" name=\"pauseresume\">";
        $content .= "<input type=\"hidden\" name=\"nr\" value=\"$file->nr\" />\n";
        
        if($file->speed >= 0) {
            $content .= "<input type=\"hidden\" name=\"ZUUL_CORE_OP\" value=\"pause\" />\n";
            $content .= "<input type=\"submit\" name=\"pause\" value=\"" . $GLOBALS["translator"]->translate("Pause") . "\" />\n";
        } else {
            $content .= "<input type=\"hidden\" name=\"ZUUL_CORE_OP\" value=\"resume\" />\n";
            $content .= "<input type=\"submit\" name=\"resume\" value=\"" . $GLOBALS["translator"]->translate("Resume") . "\" />\n";
        }
        $content .= "</form>\n</td>\n";
        $content .= "<td>\n<form method=\"post\" action=\"index.php\" name=\"cancel\">\n";
        $content .= "<input type=\"hidden\" name=\"nr\" value=\"$file->nr\" />\n";
        
        $content .= "<input type=\"hidden\" name=\"ZUUL_CORE_OP\" value=\"cancel\" />\n";
        $content .= "<input type=\"submit\" name=\"cancel\" value=\"" . $GLOBALS["translator"]->translate("Cancel") . "\" />\n";
        $content .= "</form>\n</td>\n";
        $content .= "</tr></table>\n";

        if ($this->settings["use_javascript"]) {
            $content .= "<script language=\"JavaScript\">\n";
            $content .= "//<!--\n";
            $content .= "function updateBox() {\n";
            $content .= "  document.forms.rename.newname.value = document.forms.rename.newnameselect.options[document.forms.rename.newnameselect.selectedIndex].text;\n";
            $content .= "}\n";
            $content .= "//-->\n";
            $content .= "</script>\n";
        }

        $content .= "<form name=\"rename\" action=\"index.php\" method=\"post\">\n";
        $content .= "<table width=\"40%\" border=\"1\" cellpadding=\"5\" cellspacing=\"0\"><tr><td>\n";
        $content .= "<table><tr><td>\n";
        $content .= "<input type=\"hidden\" name=\"nr\" value=\"$file->nr\" />\n";
        $content .= "<input type=\"hidden\" name=\"ZUUL_CORE_OP\" value=\"rename\" />\n";
        $content .= "<select name=\"newnameselect\"";
        if ($this->settings["use_javascript"]) {
            $content .= " onChange=\"javascript:updateBox()\"";
        }
        $content .= ">\n";
        foreach($file->names as $name) {
            $content .= "<option>" . trim($name) . "</option>\n";
        }
        $content .= "</select></td><td>\n";
        if ($this->settings["use_javascript"]) {
            $content .= "&nbsp;\n";
        } else {
            $content .= "<input type=\"submit\" name=\"renameselect\" value=\"" . $GLOBALS["translator"]->translate("Rename") . "\" />\n";
        }
        $content .= "</td></tr>\n";
        $content .= "<tr><td><input style=\"width: 100%\" type=\"text\" maxlength=\"200\" name=\"newname\" value=\"" . trim($file->names[0]) . "\" /></td>\n";
        $content .= "<td><input type=\"submit\" name=\"rename\" value=\"" . $GLOBALS["translator"]->translate("Rename") . "\" /></td>\n";
        $content .= "</tr></table>\n";
        $content .= "</td></tr></table>\n";
        $content .= "</form>\n";

        $content .= "</div>\n";

        $content .= $GLOBALS["translator"]->translate("No. of Sources") . ": $numsources<br />\n";

        if($file->network == "Donkey") {
            $content .= $GLOBALS["translator"]->translate("Link") . ": <a href=\"" . $file->getED2KLink() . "\">" . $file->getED2KLink() . "</a><br />\n";
            $content .= $GLOBALS["translator"]->translate("MD4 Hash") . ": $file->md4hash<br />\n";
        }
        $content .= $GLOBALS["translator"]->translate("Size") . ": $file->size Bytes (" . $file->hrSize() . ")<br />\n";
        $content .= $GLOBALS["translator"]->translate("Downloaded") . ": $file->downloaded Bytes (" . $file->hrDownloaded() . ")<br />\n";
        //$content .= $GLOBALS["translator"]->translate("Priority") . ": $file->priority<br />";
        $content .= $GLOBALS["translator"]->translate("Download Speed") . ": ";
        if($file->speed >= 0) { $content .= "$file->speed<br />\n"; }
        else { $content .= $GLOBALS["translator"]->translate("Paused") . "<br />\n"; }
        $content .= $GLOBALS["translator"]->translate("Download Time Remaining at Current Speed") . ": $file->eta seconds (" . $file->hrETA() . ")<br />\n";
        $content .= $GLOBALS["translator"]->translate("Days in Download List") . ": $file->daysinlist<br />\n";
        $content .= $GLOBALS["translator"]->translate("Days Since Last Seen") . ": $file->lastseen<br />\n";

        // Display chunks
        $cellwidth = 100/strlen($file->chunks);
        $content .= "<div align=\"center\">\n";
        $content .= "<strong>" . $GLOBALS["translator"]->translate("Chunk Information") . "</strong><br />\n";
        $content .= "<em>\n";
        $first = TRUE;
        foreach (array("missing", "downloading", "finished", "verified") as $chunkinfo) {
            if ($first) {
                $first = FALSE;
            } else {
                $content .= " | \n";
            }
            /* ------------- added by tokai */
                 
               switch ($chunkinfo)
               {
                  case "missing":     $cbgcolor="#ff0000";break;
                  case "downloading": $cbgcolor="#ffff00";break;
                  case "finished":    $cbgcolor="#00ff00";break;
                  case "verified":    $cbgcolor="#0000ff";break;
                  default:            $cbgcolor="#ffffff";
               }

            /* ------- */
            $content .= "<span class=\"chunk_" . $chunkinfo . "\"><font color=\"$cbgcolor\">";
            $content .= $GLOBALS["translator"]->translate(ucwords($chunkinfo));
            $content .= "</font></span>";
        }
        $content .= "</em><br /><br />\n";
        $content .= "<table width=\"75%\" border=\"1\" cellspacing=\"0\" cellpadding=\"0\"><tr><td>\n";
        $content .= "<table border=\"0\" cellspacing=\"0\" width=\"100%\"><tr>\n";
        for ($i=0; $i<strlen($file->chunks); $i++) {
            $content .= "<td width=\"$cellwidth%\" class=\"chunk_";
            switch($file->chunks[$i]) {
                case "0":
                    $content .= "missing\" bgcolor=\"#ff0000";
                    break;
                case "1":
                    $content .= "downloading\" bgcolor=\"#ffff00";
                    break;
                case "2":
                    $content .= "finished\" bgcolor=\"#00ff00";
                    break;
                case "3":
                    $content .= "verified\" bgcolor=\"#0000ff";
                    break;
                default:
                    // Note: this doesn't mean that the chunk is invalid,
                    // it means that we got a number which shouldn't exist.
                    $content .= "invalid";
            }
            $content .= "\">&nbsp;</td>\n";
        }
        $content .= "</tr></table></td></tr></table></div>\n";

        $content .= "<br />\n" . $GLOBALS["translator"]->translate("Other Names For This File") . ": \n<ul>\n";
        foreach($file->names as $name) {
            $content .= "<li>$name</li>\n";
        }
        $content .= "</ul>\n";

        $content .= "<br />\n" . $GLOBALS["translator"]->translate("Sources For This File") . ": \n<ul>\n";
    if($file->sources) {
            foreach($file->sources as $source) {
                $content .= "<li>$source</li>\n";
            }
    } else {
        $content .= "<li>" . $GLOBALS["translator"]->translate("None (empty)") . "</li>\n";
    }
        $content .= "</ul>\n";

        return $content;
    }// END FUNC _vddetails


    /**
    * Pauses file download
    *
    * @author   Don Seiler <don@NOSPAM.seiler.us>
    */
    function _pause()
    {
        if(isset($_REQUEST["nr"]) && is_numeric($_REQUEST["nr"]))
            $mldarray = $this->mldonkey("pause " . $_REQUEST["nr"]);

        // mldonkey doesn't print anything meaningful for us
        $content = "<p>" . $GLOBALS["translator"]->translate("Download successfully paused.") . "</p>\n";

        $content .= $this->_vd();

        return $content;

    }// END FUNC _pause


    /**
    * Resumes file download
    *
    * @author   Don Seiler <don@NOSPAM.seiler.us>
    */
    function _resume()
    {
        if(isset($_REQUEST["nr"]) && is_numeric($_REQUEST["nr"]))
            $mldarray = $this->mldonkey("resume " . $_REQUEST["nr"]);

        // mldonkey doesn't print anything meaningful for us
        $content = "<p>" . $GLOBALS["translator"]->translate("Download successfully resumed.") . "</p>\n";

        $content .= $this->_vd();

        return $content;

    }// END FUNC _resume


    /**
    * Stops file download
    *
    * @author   Don Seiler <don@NOSPAM.seiler.us>
    */
    function _cancel()
    {
        $content = NULL;
        if(isset($_REQUEST["ZUUL_CANCEL"])) {
            if(isset($_REQUEST["nr"]) && is_numeric($_REQUEST["nr"]))
                $mldarray = $this->mldonkey("cancel " . $_REQUEST["nr"]);
    
            // mldonkey doesn't print anything meaningful for us
            $content .= "<p>" . $GLOBALS["translator"]->translate("Download successfully cancelled.") . "</p>\n";
    
            $content .= $this->_vd();
        } elseif (isset($_REQUEST["ZUUL_NOCANCEL"])) {
            $content .= "<p>" . $GLOBALS["translator"]->translate("Download was NOT cancelled.") . "</p>\n"; 
            $content .= $this->_vd();
        } else {
            // Print confirmation page
            $content .= "<div align=\"center\">\n";
            $content .= "<form name=\"cancelform\" action=\"index.php\" method=\"post\">\n";
            $content .= "<input type=\"hidden\" name=\"ZUUL_CORE_OP\" value=\"cancel\" />\n";
            $content .= "<input type=\"hidden\" name=\"nr\" value=\"" . $_REQUEST["nr"] . "\" />\n";
            $content .= "<b>" . $GLOBALS["translator"]->translate("Are you sure you want to cancel this download?") . "</b><br />\n";
            $content .= "<input type=\"submit\" name=\"ZUUL_CANCEL\" value=\"" . $GLOBALS["translator"]->translate("Yes") . "\" />\n";
            $content .= "&nbsp;&nbsp;&nbsp;\n";
            $content .= "<input type=\"submit\" name=\"ZUUL_NOCANCEL\" value=\"" . $GLOBALS["translator"]->translate("No") . "\" />\n";
            $content .= "</form>\n</div>\n";
        }

        return $content;

    }// END FUNC _cancel


    /**
    * Renames a file
    *
    * @author   CJ Kucera <zuul@NOSPAM.apocalyptech.com>
    */
    function _rename()
    {
        if(isset($_REQUEST["nr"]) && is_numeric($_REQUEST["nr"]) && isset($_REQUEST["newname"])) {
            if (isset($_REQUEST["renameselect"])) {
                $newname = $_REQUEST["newnameselect"];
            } else {
                $newname = $_REQUEST["newname"];
            }
            if (get_magic_quotes_gpc())
            {
                $newname = stripslashes($newname);
            }
            $mldarray = $this->mldonkey("rename " . $_REQUEST["nr"] . " \"" . $newname . "\"");
        }

        // mldonkey doesn't print anything meaningful for us
        $content = "<p>" . $GLOBALS["translator"]->translate("Download successfully renamed.") . "</p>\n";

        $content .= $this->_vd();

        return $content;

    }// END FUNC _rename


    /**
    * Allows user to make command-line calls to mldonkey
    *
    * @author   Don Seiler <don@NOSPAM.seiler.us>
    */
    function _commandline()
    {
        $content = NULL;
        if(isset($_REQUEST["command"])) {
            $mldarray = $this->mldonkey(trim($_REQUEST["command"]));
            $content .= "<tt>\n";
            foreach($mldarray as $line) {
                $line = str_replace("<", "&lt;", $line);
                $line = str_replace(">", "&gt;", $line);
                $content .= "$line<br />\n";
            }
            $content .= "</tt>\n";

        } else {
            $content .= "<div align=\"center\">\n";
            $content .= $GLOBALS["translator"]->translate("Use ? for help") . "<br />\n";
            $content .= "</div>\n";
        }

        // Print form to take command
        $content .= "<div align=\"center\">\n";
        $content .= "<form method=\"post\" action=\"index.php\">\n";
        $content .= "<input type=\"hidden\" name=\"ZUUL_CORE_OP\" value=\"commandline\" />\n";
        $content .= "<input type=\"text\" name=\"command\" size=\"50\"/>\n";
        $content .= "<input type=\"submit\" name=\"sendcommand\" value=\"" . $GLOBALS["translator"]->translate("Send Command") . "\" />\n";
        $content .= "</form>\n<br /><br />\n";
        $content .= "</div>\n";

        return $content;
    }// END FUNC _commandline


    /**
    * Handles MLdonkey searches
    *
    * @author   Don Seiler <don@NOSPAM.seiler.us>
    */
    function _s()
    {
        $content = NULL;

        if(isset($_REQUEST["keywords"]))
            $this->mldonkey("s " . trim($_REQUEST["keywords"]));

        // Print form to take search keyworkds
        $content .= "<div align=\"center\">\n";
        $content .= "<form method=\"post\" action=\"index.php\">\n";
        $content .= "<input type=\"hidden\" name=\"ZUUL_CORE_OP\" value=\"s\" />\n";
        $content .= "<input type=\"text\" name=\"keywords\" size=\"50\"/>\n";
        $content .= "<input type=\"submit\" name=\"search\" value=\"" . $GLOBALS["translator"]->translate("Search") . "\" />\n";
        $content .= "</form>\n<br /><br />\n";
        $content .= "</div>\n";

        $content .= $this->_vs();

        return $content;

    }// END FUNC _s



    /**
    * Views MLdonkey searches
    *
    * @author   Don Seiler <don@NOSPAM.seiler.us>
    */
    function _vs()
    {
        $content = NULL;
        $mldarray = $this->mldonkey("vs");

        $c = 0;
        $content .= "<div align=\"center\">\n";
        $content .= "<table border=\"0\">\n";
        $content .= "<tr class=\"bg_dark\">\n";
        $content .= "<th>" . $GLOBALS["translator"]->translate("No.") . "</th>\n";
        $content .= "<th>" . $GLOBALS["translator"]->translate("Search Terms") . "</th>\n";
        $content .= "<th>" . $GLOBALS["translator"]->translate("Inquired") . "</th>\n";
        $content .= "<th>" . $GLOBALS["translator"]->translate("View") . "</th>\n";
        $content .= "<th>" . $GLOBALS["translator"]->translate("Forget") . "</th>\n";
        $content .= "</tr>\n";
        foreach($mldarray as $line) {

            $search = new ZUUL_Search($line);
            if(isset($search)) {
                // We have a search result
                $content .= "<tr class=\"";
                if($c%2 == 0) $content .= "list";
                else $content .= "alt_list";
                $content .= "\">\n";
                $content .= "<td>$search->searchnum</td>\n"; 
                $content .= "<td>$search->terms</td>\n"; 
                $content .= "<td>$search->inquired</td>\n"; 
                $content .= "<td>[ <a href=\"index.php?ZUUL_CORE_OP=vr&searchnum=$search->searchnum\">" . $GLOBALS["translator"]->translate("View Results") . "</a> ]</td>\n";
                $content .= "<td>[ <a href=\"index.php?ZUUL_CORE_OP=forget&searchnum=$search->searchnum\">" . $GLOBALS["translator"]->translate("Forget Search") . "</a> ]</td>\n";
                $content .= "</tr>\n";
                $c++;
            }
        }
        if(empty($c))
            $content .= "<tr>\n<td colspan=\"5\" align=\"center\">" . $GLOBALS["translator"]->translate("No searches in the system.") . "</td>\n</tr>\n";
        $content .= "</table>\n</div>\n";

        return $content;

    }// END FUNC _vs


    /**
    * Displays search results
    *
    * @author   Don Seiler <don@NOSPAM.seiler.us>
    */
    function _vr()
    {
        $content = NULL;

        if(!isset($_REQUEST["searchnum"])) {
            $content .= "<div class=\"error\">" . $GLOBALS["translator"]->translate("No search number specified.") . "</div>\n";
        } else {
            if(isset($_REQUEST["ZUUL_SORTKEY"]))
                $sortkey = $_REQUEST["ZUUL_SORTKEY"];
            else
                $sortkey = "nr";
    
            if(isset($_REQUEST["ZUUL_SORTORDER"]))
                $sortorder = $_REQUEST["ZUUL_SORTORDER"];
            else
                $sortorder = "ASC";

            $headers = array();
            $headers["action"]          = "";
            $headers["nr"]              = $GLOBALS["translator"]->translate("No.");
            $headers["network"]         = $GLOBALS["translator"]->translate("Network");
            $headers["name"]            = $GLOBALS["translator"]->translate("Name");
            $headers["size"]            = $GLOBALS["translator"]->translate("Size");
            $headers["availability"]    = $GLOBALS["translator"]->translate("Availability");

            $mldarray = $this->mldonkey("vr " . $_REQUEST["searchnum"]);

            $c = 0;
            $content .= $this->_checkboxJavascript();
            $content .= "<form name=\"dform\" action=\"index.php\" method=\"post\">\n";
            $content .= "<input type=\"hidden\" name=\"ZUUL_CORE_OP\" value=\"d\" />\n";
            $content .= "<input type=\"hidden\" name=\"searchnum\" value=\"" . trim($_REQUEST["searchnum"]) . "\" />\n";
            $content .= "<div align=\"center\">\n";
            $content .= "<table border=\"0\">\n";
            $content .= "<tr>\n<td colspan=\"5\">\n<input type=\"submit\" name=\"download\" value=\"" . $GLOBALS["translator"]->translate("Download Selected Items") . "\" />&nbsp;" . $this->_checkboxLinks("dform") . "\n</td>\n</tr>\n";
            $content .= "<tr class=\"bg_dark\">\n";
            foreach($headers as $key=>$value)
                if($key != "action")
                    $content .= "<th><a class=\"header\" href=\"index.php?ZUUL_CORE_OP=vr&searchnum=" . $_REQUEST["searchnum"] . "&ZUUL_SORTKEY=$key&ZUUL_SORTORDER=ASC\"></a>$value<a class=\"header\" href=\"index.php?ZUUL_CORE_OP=vr&searchnum=" . $_REQUEST["searchnum"] . "&ZUUL_SORTKEY=$key&ZUUL_SORTORDER=DESC\"></a></th>\n";
                else
                    $content .= "<th>$value</th>\n";
            $content .= "</tr>\n";

            $results = array();
            $unsorted = ZUUL_SearchResult::getResults($mldarray);
            foreach($unsorted as $result) {
                if(isset($result)) {
                    $key = NULL;
                    switch($sortkey) {
                        case "nr":
                            $key = $result->resultnum;
                            break;
                        case "network":
                            $key = $result->network;
                            break;
                        case "name":
                            $key = $result->name;
                            break;
                        case "size":
                            $key = $result->size;
                            break;
                        case "availability":
                            $key = $result->availability;
                            break;
                    }
                    $results[$key][] = $result;
                }
            }
            if($sortorder == "DESC")
                krsort($results);
            else
                ksort($results);
            reset($results);

            foreach($results as $resultlevel) {
                foreach($resultlevel as $result) {
                    if(isset($result)) {
                        $content .= "<tr class=\"";
                        if($c%2 == 0) $content .= "list";
                        else $content .= "alt_list";
                        $content .= "\">\n";
                        $content .= "<td>\n<input type=\"checkbox\" name=\"resultnum[]\" value=\"$result->resultnum\" />\n";
                        $content .= "<input type=\"hidden\" name=\"name_" . $result->resultnum . "\" value=\"$result->name\" />\n</td>\n";
                        $content .= "<td>$result->resultnum</td>\n"; 
                        $content .= "<td>$result->network</td>\n"; 
                        if($result->network == "Donkey")
                            $content .= "<td><a href=\"" . ZUUL_File::ed2kLink($result->name, $result->size, $result->md4hash) . "\">$result->name</a></td>\n"; 
                        else
                            $content .= "<td>$result->name</td>\n"; 
                        $content .= "<td>" . ZUUL_File::hrSizes($result->size) . "</td>\n"; 
                        $content .= "<td>$result->availability</td>\n"; 
                        $content .= "</tr>\n";
                        $c++;
                    }
                }
            }
            $content .= "<tr>\n<td colspan=\"5\">\n<input type=\"submit\" name=\"download\" value=\"" . $GLOBALS["translator"]->translate("Download Selected Items") . "\" />&nbsp;" . $this->_checkboxLinks("dform") . "\n</td>\n</tr>\n";
            $content .= "</table>\n</div>\n";
            $content .= "</form>\n";
        }

        return $content;

    }// END FUNC _vr


    /**
    * Forgets search results
    *
    * @author   Don Seiler <don@NOSPAM.seiler.us>
    */
    function _forget()
    {
        $content = NULL;

        if(!isset($_REQUEST["searchnum"])) {
            $content .= "<div class=\"error\">" . $GLOBALS["translator"]->translate("No search number specified.") . "</div>";
        } else {
            $mldarray = $this->mldonkey("forget " . $_REQUEST["searchnum"]);

            // MLdonkey doesn't return anything useful
            $content .= "<p>" . $GLOBALS["translator"]->translate("Search forgotten.") . "</p>\n";

        }

        $content .= $this->_s();
        return $content;

    }// END FUNC _forget


    /**
    * Downloads search result
    *
    * @author   Don Seiler <don@NOSPAM.seiler.us>
    */
    function _d()
    {
        $content = NULL;
        if(!isset($_REQUEST["resultnum"])) {
            $content .= "<div class=\"error\">" . $GLOBALS["translator"]->translate("No search result number specified.") . "</div>\n";
        } else {
            if(is_array($_REQUEST["resultnum"])) {
                $commandarray = array();
                foreach($_REQUEST["resultnum"] as $rn) {
                    $commandarray[] = "d $rn";
                }

                $resultsarray = $this->mldonkey($commandarray);
                $i = 0;
                foreach($_REQUEST["resultnum"] as $rn) {
                    $content .= "Downloading " . $_REQUEST["name_" . $rn] . " ... ";
                    $mldarray = $resultsarray[$i];
                    $i++;
    
                    // Need to decide how best to report this for bulk operations
                    $started = FALSE;
                    $already = FALSE;
                    foreach($mldarray as $line) {
                        if(preg_match("/^download started/", $line)) {
                            $started = TRUE;
                            break;
                        }
                        if(preg_match("/File already downloaded/", $line)) {
                            $already = TRUE;
                            break;
                        }
                    }
        
                    if($started) {
                        $content .= $GLOBALS["translator"]->translate("Download started.") . "<br />\n";
                    } elseif($already) {
                        $content .= "<span class=\"error\">" . $GLOBALS["translator"]->translate("File already downloaded.") . "  <em><a href=\"index.php?ZUUL_CORE_OP=force_download\">(" . $GLOBALS["translator"]->translate("Force Download") . ")</em></span>\n<br />\n";
                    } else {
                        $content .= "<span class=\"error\">" . $GLOBALS["translator"]->translate("There was an error starting the download.") . "</span>\n<br />\n";
                    }
                }
            } else {
                $mldarray = $this->mldonkey("d " . $_REQUEST["resultnum"]);
                $started = FALSE;
                foreach($mldarray as $line) {
                    if(preg_match("/^download started/", $line)) {
                        $started = TRUE;
                        break;
                    }
                }
    
                if($started)
                    $content .= $GLOBALS["translator"]->translate("Download started.") . "\n<br />\n";
            }
        }
        $content .= $this->_vr();
        return $content;

    }// END FUNC _d


    /**
    * Downloads link
    *
    * @author   Don Seiler <don@NOSPAM.seiler.us>
    */
    function _dllink()
    {
        $content = NULL;
        if(isset($_REQUEST["ed2klinks"])) {
            $explodestring = "\n";
            if (strpos($links, "\r\n") !== false) {
                $explodestring = "\r\n";
            } elseif (strpos($links, "\r") !== false) {
                $explodestring = "\r";
            }
            $links = explode($explodestring, $_REQUEST["ed2klinks"]);
            $commandarray = array();
            foreach($links as $link) {
                $link = trim($link);
                if(empty($link))
                    continue;
                $commandarray[] = "dllink $link";
            }

            $resultsarray = $this->mldonkey($commandarray);

            $i = 0;
            foreach($links as $link) {
                $link = trim($link);
                if(empty($link))
                    continue;

                if (preg_match("/ed2k:\/\/\|file\|(.+)\|(\d+)\|(\w+)\|/i", $link, $pregarray)) {
                    $displayname = $pregarray[1];
                } else {
                    $displayname = $link;
                }

                $content .= "$displayname - ";

                $mldarray = $resultsarray[$i];
                $i++;
                $e = FALSE;
                foreach($mldarray as $line) {
                    // Look for goodies or baddies
                    if(preg_match("/File already downloaded/", $line)) {
                        $content .= "<span class=\"error\">" . $GLOBALS["translator"]->translate("File already downloaded.") . "  <em><a href=\"index.php?ZUUL_CORE_OP=force_download\">(" . $GLOBALS["translator"]->translate("Force Download") . ")</a></em></span><br />\n";
                        $e = TRUE;
                        break;
                    } elseif(preg_match("/Unable to match URL/", $line)) {
                        $content .= "<span class=\"error\">" . $GLOBALS["translator"]->translate("Unable to match URL.") . "</span><br />\n";
                        $e = TRUE;
                        break;
                    }
                }
                if(!$e)
                    $content .= $GLOBALS["translator"]->translate("Download started.") . "<br />\n";
            }

            if(!$e)
                $content .= $GLOBALS["translator"]->translate("File(s) successfully queued for download.") . "<br />\n";
            $content .= $this->_vd();

        } else {
            // Print form to add ed2k links
            $content .= "<div align=\"center\">\n";
            $content .= "<form method=\"post\" action=\"index.php\">\n";
            $content .= "<input type=\"hidden\" name=\"ZUUL_CORE_OP\" value=\"dllink\" />\n";
            $content .= $GLOBALS["translator"]->translate("Paste ED2K Links") . ":\n<br />\n";
            $content .= "<textarea name=\"ed2klinks\" cols=\"50\" rows=\"8\" /></textarea><br />\n";
            $content .= "<input type=\"submit\" name=\"dllink\" value=\"" . $GLOBALS["translator"]->translate("Download") . "\" />\n";
            $content .= "</form><br /><br />\n";
            $content .= "<form method=\"post\" enctype=\"multipart/form-data\" action=\"index.php\">\n";
            $content .= $GLOBALS["translator"]->translate("Upload File of ED2K Links") . ":\n";
            $content .= "<input type=\"hidden\" name=\"ZUUL_CORE_OP\" value=\"dllinks\" />\n";
            $content .= "<input type=\"file\" name=\"ed2klinkfile\" />\n ";
            $content .= "<input type=\"submit\" name=\"dllink\" value=\"" . $GLOBALS["translator"]->translate("Download") . "\" />\n";
            $content .= "</form>\n";
            $content .= "</div>\n";
        }

        return $content;
    }// END FUNC _dllink


    /**
    * Uploads file containing ed2k links and sends it to mldonkey
    *
    * @author   Don Seiler <don@NOSPAM.seiler.us>
    */
    function _dllinks()
    {
        $content = NULL;
        if(!isset($_FILES["ed2klinkfile"]["error"])) {
            if(is_uploaded_file($_FILES["ed2klinkfile"]["tmp_name"])) {
                $mldarray = $this->mldonkey("dllinks " . $_FILES["ed2klinkfile"]["tmp_name"]);

                $content .= $this->_vd();
            } else {
                $content .= "<div class=\"error\">" . $GLOBALS["translator"]->translate("There was an error transferring the file.") . "</div>\n";
            }
        } else {
            $content .= "<div class=\"error\">\nERROR CODE " . $_FILES["ed2klinkfile"]["error"] . ": ";
            switch($_FILES["ed2klinkfile"]["error"]) {
                case UPLOAD_ERR_INI_SIZE:
                case UPLOAD_ERR_FORM_SIZE:
                    $content .= $GLOBALS["translator"]->translate("The uploaded file exceeds the upload max filesize.");
                    break;
                case UPLOAD_ERR_PARTIAL:
                    $content .= $GLOBALS["translator"]->translate("The uploaded file was only partially uploaded.");
                    break;
                case UPLOAD_ERR_NO_FILE:
                    $content .= $GLOBALS["translator"]->translate("No file was uploaded");
                    break;
            }
            $content .= "\n</div>\n";
        }

        return $content;

    }// END FUNC _dllinks


    /**
    * Forces a download
    *
    * @author   CJ Kucera <zuul@NOSPAM.apocalyptech.com>
    */
    function _force_download()
    {
        $this->mldonkey("force_download");
        $content = $GLOBALS["translator"]->translate("Download Forced");
        $content .= $this->_vd();
        return $content;
    }// END FUNC _force_download


    /**
    * Connects to server
    * 
    * @author   Don Seiler <don@NOSPAM.seiler.us>
    */
    function _c()
    {
        $content = NULL;

        if(isset($_REQUEST["nr"]) && is_numeric($_REQUEST["nr"]))
            $mldarray = $this->mldonkey("c " . $_REQUEST["nr"]);

        // mldonkey doesn't print anything meaningful for us
        $content = "<p>" . $GLOBALS["translator"]->translate("Connected to server successfully.") . "</p>\n";
        $content .= $this->_vm(TRUE);

        return $content;

    }// END FUNC _c


    /**
    * Disconnects from server
    * 
    * @author   Don Seiler <don@NOSPAM.seiler.us>
    */
    function _x()
    {
        $content = NULL;

        if(isset($_REQUEST["nr"]) && is_numeric($_REQUEST["nr"]))
            $mldarray = $this->mldonkey("x " . $_REQUEST["nr"]);

        // mldonkey doesn't print anything meaningful for us
        $content = "<p>" . $GLOBALS["translator"]->translate("Disconnected from server successfully.") . "</p>\n";

        $content .= $this->_vm();

        return $content;

    }// END FUNC _x


    /**
    * Lists connected servers
    *
    * @author   Don Seiler <don@NOSPAM.seiler.us>
    */
    function _vm($a = FALSE)
    {
        if(isset($_REQUEST["ZUUL_SORTKEY"]))
            $sortkey = $_REQUEST["ZUUL_SORTKEY"];
        else
            $sortkey = "files";

        if(isset($_REQUEST["ZUUL_SORTORDER"]))
            $sortorder = $_REQUEST["ZUUL_SORTORDER"];
        else
            $sortorder = "DESC";

        $headers = array();
        $headers["action"]  = "";
        $headers["nr"]      = $GLOBALS["translator"]->translate("No.");
        $headers["ip"]      = $GLOBALS["translator"]->translate("IP Address");
        $headers["name"]    = $GLOBALS["translator"]->translate("Name");
        $headers["files"]   = $GLOBALS["translator"]->translate("No. of Files");
        $headers["users"]   = $GLOBALS["translator"]->translate("No. of Users");
        $headers["status"]  = $GLOBALS["translator"]->translate("Status");
        $headers["network"] = $GLOBALS["translator"]->translate("Network");

        $content = NULL;
        $content .= $this->_checkboxJavascript();
        $content .= "<form name=\"vm\" action=\"index.php\" method=\"post\">\n";
        $content .= "<div align=\"center\"><table border=\"0\">\n";
        $content .= "<tr><td colspan=\"8\">" . $this->_checkboxLinks("vm") . "</td></tr>\n";
        $content .= "<tr class=\"bg_dark\">\n";
        foreach($headers as $key=>$value) {
            if($a) { $op = "vma"; } else { $op = "vm"; }

            if($key != "action")
                $content .= "<th><a class=\"header\" href=\"index.php?ZUUL_CORE_OP=$op&ZUUL_SORTKEY=$key&ZUUL_SORTORDER=ASC\"></a>$value<a class=\"header\" href=\"index.php?ZUUL_CORE_OP=$op&ZUUL_SORTKEY=$key&ZUUL_SORTORDER=DESC\"></a></th>\n";
            else
                $content .= "<th>$value</th>\n";
        }
        $content .= "</tr>\n";

        if($a)
            $mldarray = $this->mldonkey("vma");
        else
            $mldarray = $this->mldonkey("vm");

        $servers = array();
        foreach($mldarray as $line) {
            $server = new ZUUL_Server($line);
            if(isset($server)) {
                $key = NULL;
                switch($sortkey) {
                    case "network":
                        $key = $server->network;
                        break;

                    case "nr":
                        $key = $server->nr;
                        break;

                    case "ip":
                        $key = $server->ip;
                        break;

                    case "name":
                        $key = $server->name;
                        break;

                    case "files":
                        $key = $server->files;
                        break;

                    case "users":
                        $key = $server->users;
                        break;

                    case "status":
                        $key = $server->status;
                        break;

                    default:
                        $key = $server->files;
                }
                $servers[$key][] = $server;
            }
        }

        // Sort
        if($sortorder == "DESC")
            krsort($servers);
        else
            ksort($servers);
        reset($servers);


        $c = 0;
        foreach($servers as $serverlevel) {
            foreach($serverlevel as $server) {
                $content .= "<tr class=\"";
                if($c%2 == 0) $content .= "list";
                else $content .= "alt_list";
                $content .= "\">\n";
                $content .= "<td><input type=\"checkbox\" name=\"nr[]\" value=\"$server->nr\" /></td>\n";
                $content .= "<td>$server->nr</td>\n";
                $content .= "<td>$server->ip</td>\n";
                $content .= "<td>$server->name</td>\n";
                $content .= "<td>$server->files</td>\n";
                $content .= "<td>$server->users</td>\n";
                $content .= "<td>$server->status</td>\n";
                $content .= "<td>$server->network</td>\n";
                $content .= "</tr>\n";
                $c++;
            }
        }
        $content .= "<tr><td colspan=\"8\">" . $this->_checkboxLinks("vm") . "</td></tr>\n";
        $content .= "<tr><td colspan=\"8\">\n";
        $content .= "<hr size=\"1\" noshade />\n";
        $content .= "</td></tr>\n";
        $content .= "<tr><td colspan=\"8\">\n";
        $content .= "<input type=\"submit\" name=\"action_connect\" value=\"" . $GLOBALS["translator"]->translate("Connect") . "\" /> \n";
        $content .= "<input type=\"submit\" name=\"action_disconnect\" value=\"" . $GLOBALS["translator"]->translate("Disconnect") . "\" /> \n";
        $content .= "<input type=\"submit\" name=\"action_remove\" value=\"" . $GLOBALS["translator"]->translate("Remove") . "\" />\n";
        $content .= "</td></tr>\n";
        if(!$a)
            $content .= "<tr>\n<th colspan=\"8\"><b><a href=\"index.php?ZUUL_CORE_OP=vma\">" . $GLOBALS["translator"]->translate("View All Servers") . "</a></b></th>\n</tr>\n";
        $content .= "</table>\n</div>\n";
        $content .= "<input type=\"hidden\" name=\"ZUUL_CORE_OP\" value=\"bulk\" />\n";
        $content .= "</form>\n";
        return $content;
    }// END FUNC _vm


    /**
    * Presents form to edit options
    *
    * @author   Don Seiler <don@NOSPAM.seiler.us>
    */
    function _vo($o = FALSE)
    {
        $sortorder = NULL;
        if(isset($_REQUEST["ZUUL_SORTORDER"]))
            $sortorder = $_REQUEST["ZUUL_SORTORDER"];

        $content = NULL;
        $options = array();
        if($o)
            $op = "voo";
        else
            $op = "vo";

        $mldarray = $this->mldonkey($op);

        $start = TRUE;
        foreach($mldarray as $line) {

            // Options end with blank line
            if(preg_match("/^$/",trim($line)))
                $start = FALSE;

            if($start) {
                $temp = explode(" ",$line);
                $length = sizeof($temp);
                $value = NULL;
                for($i = 2; $i < $length; $i++) {
                    if($i > 2)
                        $value .= " ";
                    $value .= $temp[$i];
                }
                $options[$temp[0]] = $value;
            }
        }

        // Sort
        if ($op == "voo" and !isset($sortorder)) {
            $sortorder = "ASC";
        }
        if(isset($sortorder)) {
            if($sortorder == "DESC")
                krsort($options);
            else
                ksort($options);
            reset($options);
        }

        // If new options were submitted, compare to old ones and set differences
        if(isset($_REQUEST["options"]) && is_array($_REQUEST["options"])) {
            // If we ever start caring about the output from the set commands,
            // this function will have to bear some rewrites.
            $commandarray = array();
            foreach($_REQUEST["options"] as $key=>$value) {
                if(trim($value) != trim($options[$key])) {
                    $commandarray[] = "set $key $value";
                    $content .= $GLOBALS["translator"]->translate("Option changed") . ": $key: $options[$key] => $value<br />";
                    $options[$key] = $value;
                }
            }
            if (count($commandarray) > 0) {
                $this->mldonkey($commandarray);
            }
        }

        $content .= "<form name=\"optionsform\" action=\"index.php\" method=\"post\">\n";
        $content .= "<input type=\"hidden\" name=\"ZUUL_CORE_OP\" value=\"$op\" />\n";
        $content .= "<div align=\"center\">\n<table border=\"0\">\n";
        $content .= "<tr class=\"bg_dark\">\n";
        $content .= "<th><a class=\"header\" href=\"index.php?ZUUL_CORE_OP=$op&ZUUL_SORTORDER=ASC\"></a>" . $GLOBALS["translator"]->translate("Key") . "<a class=\"header\" href=\"index.php?ZUUL_CORE_OP=$op&ZUUL_SORTORDER=DESC\"></a> </th>\n";
        $content .= "<th>" . $GLOBALS["translator"]->translate("Value") . "</th>\n";
        $content .= "</tr>\n";

        $c = 0;
        foreach($options as $key=>$value) {
            $content .= "<tr class=\"";
            if($c%2 == 0) $content .= "list";
            else $content .= "alt_list";
            $content .= "\">\n";
            $content .= "<td>$key</td>\n";
            $content .= "<td><input type=\"text\" size=\"10\" name=\"options[$key]\" value=\"$value\" /></td>\n</tr>\n";
            $c++;
        }

        $content .= "<tr>\n<td colspan=\"2\" align=\"right\">\n<input type=\"submit\" name=\"submitoptions\" value=\"" . $GLOBALS["translator"]->translate("Save Options") . "\" />\n</td>\n</tr>\n";

        if(!$o)
            $content .= "<tr>\n<th colspan=\"2\">\n<b><a href=\"index.php?ZUUL_CORE_OP=voo\">" . $GLOBALS["translator"]->translate("View All Options") . "</a></b>\n</th>\n</tr>\n";

        $content .= "</table>\n</div>\n";
        $content .= "</form>\n";

        return $content;
    }// END FUNC _vo


    /**
    * Saves mldonkey options to disk
    *
    * @author   Don Seiler <don@NOSPAM.seiler.us>
    */
    function _save()
    {
        $content = NULL;
        $mldarray = $this->mldonkey("save");
        foreach($mldarray as $line)
            if(preg_match("/^saved/", $line)) {
                $content .= "<p>" . $GLOBALS["translator"]->translate("Configuration files saved.") . "</p>\n";
                break;
            } 

        return $content;
    }// END FUNC _save


    /**
    * Kills mldonkey process
    *
    * @author   Don Seiler <don@NOSPAM.seiler.us>
    */
    function _kill()
    {
        if(isset($_REQUEST["ZUUL_KILL"])) {
            $this->mldonkey("kill\n");
            echo "<br />" . $GLOBALS["translator"]->translate("MLdonkey process terminated.") . "<br />\n";
            flush();
        } elseif (isset($_REQUEST["ZUUL_NOKILL"])) {
            return $GLOBALS["translator"]->translate("You have chosen wisely."); 
        } else {
            // Print confirmation page
            $content = "<div align=\"center\">\n";
            $content .= "<form name=\"killform\" action=\"index.php\" method=\"post\">\n";
            $content .= "<input type=\"hidden\" name=\"ZUUL_CORE_OP\" value=\"kill\" />\n";
            $content .= "<b>" . $GLOBALS["translator"]->translate("Are you sure you want to kill MLdonkey?") . "</b><br />\n";
            $content .= "<input type=\"submit\" name=\"ZUUL_KILL\" value=\"" . $GLOBALS["translator"]->translate("Yes") . "\" />\n";
            $content .= "&nbsp;&nbsp;&nbsp;\n";
            $content .= "<input type=\"submit\" name=\"ZUUL_NOKILL\" value=\"" . $GLOBALS["translator"]->translate("No") . "\" />\n";
            $content .= "</form>\n</div>\n";

            return $content;
        }
    }// END FUNC _kill


    /**
    * Starts mldonkey process
    *
    * @author   Don Seiler <don@NOSPAM.seiler.us>
    */
    function _start()
    {
        exec($this->settings["mldonkey_startup"]);
        echo $GLOBALS["translator"]->translate("Starting MLdonkey");
        flush();
        while(!$this->alive()) {
            echo ".";
            flush();
            sleep(1);
        }
        echo $GLOBALS["translator"]->translate("Finished!");
    }// END FUNC _start



    /**
    * Makes test connection to see if mldonkey is running
    *
    * @author   Don Seiler <don@NOSPAM.seiler.us>
    */
    function alive()
    {
        if($open = fsockopen($this->settings["mldonkey_host"], $this->settings["mldonkey_port"])) {
            fclose($open);
            return TRUE;
        } else {
            return FALSE;
        }
    }// END FUNC alive


    /**
    * Sets file priority
    *
    * @author   Don Seiler <don@NOSPAM.seiler.us>
    */
    function _priority()
    {
        $content = NULL;
        if(isset($_REQUEST["priority"]) && isset($_REQUEST["nr"])) {
            $mldarray = $this->mldonkey("priority =" . $_REQUEST["priority"] . " " . $_REQUEST["nr"]);
            foreach ($mldarray as $line) {
                if(preg_match("/^Done/",$line)) {
                    $content .= "<p>" . $GLOBALS["translator"]->translate("Priority successfully updated.") . "</p>\n";
                    break;
                }
            }
            $content .= $this->_vddetails();
        } else {
            $content = "<div class=\"error\">" . $GLOBALS["translator"]->translate("No priority or file number given.") . "</div>\n"; 
        }
        return $content;
    }// END FUNC _priority


    /**
    * Commits files (moves them to incoming directory)
    *
    * @author   Don Seiler <don@NOSPAM.seiler.us>
    */
    function _commit()
    {
        $content = NULL;
        if(isset($_REQUEST["ZUUL_COMMIT"])) {

            $mldarray = $this->mldonkey("commit");

            $committed = FALSE;
            foreach ($mldarray as $line) {
                if(preg_match("/^Commited/", $line)) {
                    $committed = TRUE;
                    break;
                }
            }

            if($committed)
                $content .= $GLOBALS["translator"]->translate("Files committed.") . "<br />\n";
            else
                $content .= "<div class=\"error\">" . $GLOBALS["translator"]->translate("There was an error committing the files.") . "</div><br />\n";

        }

        if(isset($_REQUEST["ZUUL_SORTKEY"]))
            $sortkey = $_REQUEST["ZUUL_SORTKEY"];
        else
            $sortkey = "name";

        if(isset($_REQUEST["ZUUL_SORTORDER"]))
            $sortorder = $_REQUEST["ZUUL_SORTORDER"];
        else
            $sortorder = "ASC";

        $mldarray = $this->mldonkey("vd");

        $finishedfiles = array();
        $finished = FALSE;
        foreach($mldarray as $line) {
            if (preg_match("/^Downloaded\s{1}[^0]{1}\d*\s{1}/", $line)) {
                // Begin of finished sections
                $finished = TRUE;
                continue;
            }

            $file = new ZUUL_File($line, FALSE, $finished);
            if(isset($file)) {
                $key = NULL;
                switch($sortkey) {
                    case "network":
                        $key = $file->network;
                        break;

                    case "nr":
                        $key = $file->nr;
                        break;

                    case "name":
                        $key = $file->name;
                        break;

                    case "percent":
                        $key = $file->percent;
                        break;

                    case "priority":
                        $key = $file->priority;
                        break;

                    case "size":
                        $key = $file->size;
                        break;

                    case "downloaded":
                        $key = $file->downloaded;
                        break;

                    case "md4hash":
                        $key = $file->md4hash;
                        break;

                    default:
                        $key = $file->speed;
                }
                $files[$key][] = $file;
            }

        }

        // Sort
        if($sortorder == "DESC")
            krsort($files);
        else
            ksort($files);
        reset($files);

        $headers = array();
        $headers["nr"]          = $GLOBALS["translator"]->translate("No.");
        $headers["name"]        = $GLOBALS["translator"]->translate("Name");
        $headers["size"]        = $GLOBALS["translator"]->translate("Size");
        $headers["md4hash"]     = $GLOBALS["translator"]->translate("MD4 Hash");
        $headers["network"]     = $GLOBALS["translator"]->translate("Network");

        $content .= "<div align=\"center\">\n";
        $content .= "<h3>" . $GLOBALS["translator"]->translate("Finished Downloads") . "</h3>\n";
        $content .= "<form name=\"commitform\" action=\"index.php\" method=\"post\">\n";
        $content .= "<input type=\"hidden\" name=\"ZUUL_CORE_OP\" value=\"commit\"/>\n";
        $content .= "<input type=\"submit\" name=\"ZUUL_COMMIT\" value=\"" . $GLOBALS["translator"]->translate("Click here to commit these files") . "\"/>\n";
        $content .= "</form>\n";
        $content .= "<table cellspacing=\"0\" cellpadding=\"2\" border=\"0\">\n";
        $content .= "<tr class=\"bg_dark\">\n";
        foreach($headers as $key=>$value)
            if($key != "action")
                $content .= "<th><a class=\"header\" href=\"index.php?ZUUL_CORE_OP=commit&ZUUL_SORTKEY=$key&ZUUL_SORTORDER=ASC\"></a>$value<a class=\"header\" href=\"index.php?ZUUL_CORE_OP=commit&ZUUL_SORTKEY=$key&ZUUL_SORTORDER=DESC\"></a></th>\n";
            else
                $content .= "<th>$value</th>\n";
        $content .= "</tr>\n";

        $c = 0;
        foreach($files as $filelevel) {
            foreach($filelevel as $file) {
                if(!$file->finished)
                    continue;
                $content .= "<tr class=\"";
                if($c%2 == 0) $content .= "list";
                else $content .= "alt_list";
                $content .= "\">\n";
                $content .= "<td>$file->nr</td>\n";
                $content .= "<td>$file->name</td>\n";
                $content .= "<td>" . $file->hrSize() . "</td>\n";
                $content .= "<td>$file->md4hash</td>\n";
                $content .= "<td>$file->network</td>\n";
                $content .= "</tr>\n";
            }
        }
        $content .= "</table>\n</div>\n";
        return $content;
    }// END FUNC _commit

    
    /**
    * Handles bulk file operations
    *
    * @author   Don Seiler <don@NOSPAM.seiler.us>
    */
    function _bulk()
    {
        $content = NULL;
        if(!(isset($_REQUEST["nr"]) && is_array($_REQUEST["nr"]))) {
            $content .= "<div class=\"error\">" . $GLOBALS["translator"]->translate("No items were selected.") . "</div>";
        } else {

            if(isset($_REQUEST["action_pause"])) {
                // Pause the array members
                $commandarray = array();
                foreach($_REQUEST["nr"] as $nr)
                    $commandarray[] = "pause $nr";
                $this->mldonkey($commandarray);

                $content .= "<p>" . $GLOBALS["translator"]->translate("Downloads successfully paused.") . "</p>\n";
            } elseif(isset($_REQUEST["action_resume"])) {
                // Resume the array members
                $commandarray = array();
                foreach($_REQUEST["nr"] as $nr)
                    $commandarray[] = "resume $nr";
                $this->mldonkey($commandarray);

                $content .= "<p>" . $GLOBALS["translator"]->translate("Downloads successfully resumed.") . "</p>\n";
            } elseif(isset($_REQUEST["action_cancel"])) {
                if(isset($_REQUEST["ZUUL_CANCEL"])) {
                    // Cancel the array members
                    $commandarray = array();
                    foreach($_REQUEST["nr"] as $nr)
                        $commandarray[] = "cancel $nr";
                    $this->mldonkey($commandarray);
    
                    // mldonkey doesn't print anything meaningful for us
                    $content .= "<p>" . $GLOBALS["translator"]->translate("Downloads successfully cancelled.") . "</p>\n";
    
                } elseif (isset($_REQUEST["ZUUL_NOCANCEL"])) {
                    $content .= "<p>" . $GLOBALS["translator"]->translate("Downloads were NOT cancelled.") . "</p>\n"; 
                } else {
                    // Print confirmation page
                    $content .= "<div align=\"center\">\n";
                    $content .= "<form name=\"cancelform\" action=\"index.php\" method=\"post\">\n";
                    $content .= "<input type=\"hidden\" name=\"ZUUL_CORE_OP\" value=\"bulk\" />\n";
                    $content .= "<input type=\"hidden\" name=\"action_cancel\" value=\"1\" />\n";
                    $filenames = array();
                    foreach($_REQUEST["nr"] as $nr) {
                        $content .= "<input type=\"hidden\" name=\"nr[]\" value=\"" . $nr . "\" />\n";
                        $filenames[] = $_REQUEST["name_" . $nr];
                    }
                    $content .= "<b>" . $GLOBALS["translator"]->translate("Are you sure you want to cancel these downloads?") . "</b><br />\n";
                    $content .= "<ul>\n";
                    foreach($filenames as $filename)
                        $content .= "<li>$filename</li>\n";
                    $content .= "</ul>\n";
                    $content .= "<input type=\"submit\" name=\"ZUUL_CANCEL\" value=\"" . $GLOBALS["translator"]->translate("Yes") . "\" />\n";
                    $content .= "&nbsp;&nbsp;&nbsp;\n";
                    $content .= "<input type=\"submit\" name=\"ZUUL_NOCANCEL\" value=\"" . $GLOBALS["translator"]->translate("No") . "\" />\n";
                    $content .= "</form>\n</div>\n";
                }

            } elseif(isset($_REQUEST["setpriority"])) {
                // Set priority of array members to $_REQUEST["priority"]
                $command = "priority =" . $_REQUEST["priority"];
                foreach($_REQUEST["nr"] as $nr)
                    $command .= " $nr";

                $this->mldonkey($command);

                $content .= "<p>" . $GLOBALS["translator"]->translate("Priorities successfully updated.") . "</p>\n";
            } elseif(isset($_REQUEST["action_connect"])) {
                // Connect to the array members
                $commandarray = array();
                foreach($_REQUEST["nr"] as $nr)
                    $commandarray[] = "c $nr";
                $this->mldonkey($commandarray);

                $content .= "<p>" . $GLOBALS["translator"]->translate("Servers successfully connected.") . "</p>\n";
            } elseif(isset($_REQUEST["action_disconnect"])) {
                // Disconnect from the array members
                $commandarray = array();
                foreach($_REQUEST["nr"] as $nr)
                    $commandarray[] = "x $nr";
                $this->mldonkey($commandarray);

                $content .= "<p>" . $GLOBALS["translator"]->translate("Servers successfully disconnected.") . "</p>\n";
            } elseif(isset($_REQUEST["action_remove"])) {
                // Remove the array members
                $command = "rem";
                foreach($_REQUEST["nr"] as $nr)
                    $command .= " $nr";

                $this->mldonkey($command);

                $content .= "<p>" . $GLOBALS["translator"]->translate("Servers successfully removed.") . "</p>\n";
            } elseif (isset($_REQUEST["render_ed2k"])) {
                // Render page of ed2k links
                $commandarray = array();
                $commandindex = array();
                $i = 0;
                foreach($_REQUEST["nr"] as $number) {
                    $commandarray[] = "vd $number";
                    $commandindex[$number] = $i;
                    $i++;
                }
                $mldarray = $this->mldonkey("set max_name_len 200", "vd", "set max_name_len ". $this->settings["default_max_name_len"], $commandarray);
                foreach($mldarray[1] as $line) {
                    $file = new ZUUL_File($line, FALSE, $finished);
                    if (in_array($file->nr, $_REQUEST["nr"])) {
                        $file->getDetails($mldarray[$commandindex[$file->nr] + 3]);
                        if($file->network == "Donkey")
                            $content .= "<a href=\"" . $file->getED2KLink() . "\">" . $file->getED2KLink() . "</a><br />\n";
                        else
                            $content .= $file->name . " is not on the Donkey network and cannot have an ED2K link.<br />\n";
                    }
                }
            }

        }

        if(!isset($_REQUEST["render_ed2k"]))
            if(isset($_REQUEST["action_remove"])
                || isset($_REQUEST["action_disconnect"])
                || isset($_REQUEST["action_connect"]))
                $content .= $this->_vm();
            elseif(!isset($_REQUEST["action_cancel"]))
                $content .= $this->_vd();
            elseif(isset($_REQUEST["action_cancel"]) && (isset($_REQUEST["ZUUL_CANCEL"]) || isset($_REQUEST["ZUUL_NOCANCEL"])))
                $content .= $this->_vd();

        return $content;
    }// END FUNC _bulk


    /**
    * Displays Javascript to do checkbox manipulations
    *
    * @author   CJ Kucera <zuul@NOSPAM.apocalyptech.com>
    */
    function _checkboxJavascript()
    {
        $content = "";
        if ($this->settings["use_javascript"]) {
            $content .= "<script language=\"Javascript\">\n";
            $content .= "//<!--\n";
            $content .= "// Based heavily on Javascript from sharereactor.com\n";
            $content .= "function allBoxes(id, value) {\n";
            $content .= "  var i=0;\n";
            $content .= "  while (id.elements[i]) {\n";
            $content .= "    if (id.elements[i].name == 'nr[]') {\n";
            $content .= "      id.elements[i].checked=value;\n";
            $content .= "    }\n";
            $content .= "    i++;\n";
            $content .= "  }\n";
            $content .= "}\n";
            $content .= "function invertBoxes(id) {\n";
            $content .= "  var i=0;\n";
            $content .= "  while (id.elements[i]) {\n";
            $content .= "    if (id.elements[i].name == 'nr[]') {\n";
            $content .= "      if (id.elements[i].checked == 1) {\n";
            $content .= "        id.elements[i].checked=0;\n";
            $content .= "      } else {\n";
            $content .= "        id.elements[i].checked=1;\n";
            $content .= "      }\n";
            $content .= "    }\n";
            $content .= "    i++;\n";
            $content .= "  }\n";
            $content .= "}\n";
            $content .= "function rangeBoxes(id) {\n";
            $content .= "  var i=0;\n";
            $content .= "  var foundchecked=0;\n";
            $content .= "  while (id.elements[i]) {\n";
            $content .= "    if (id.elements[i].name == 'nr[]') {\n";
            $content .= "      if (id.elements[i].checked == 1) {\n";
            $content .= "        if (foundchecked == 1) {\n";
            $content .= "          return;\n";
            $content .= "        }\n";
            $content .= "        foundchecked = 1;\n";
            $content .= "      } else if (foundchecked == 1) {\n";
            $content .= "        id.elements[i].checked = 1;\n";
            $content .= "      }\n";
            $content .= "    }\n";
            $content .= "    i++;\n";
            $content .= "  }\n";
            $content .= "}\n";
            $content .= "//-->\n";
            $content .= "</script>\n";
        }
        return $content;
    }// END FUNC _checkboxJavascript


    /**
    * Displays links to do checkbox manipulations
    *
    * @author   CJ Kucera <zuul@NOSPAM.apocalyptech.com>
    */
    function _checkboxLinks($formname)
    {
        $content = "";
        if ($this->settings["use_javascript"]) {
            $content .= "<a href=\"javascript:allBoxes(document.forms.$formname, 1);\">" . $GLOBALS["translator"]->translate("All") . "</a> |\n";
            $content .= "<a href=\"javascript:allBoxes(document.forms.$formname, 0);\">" . $GLOBALS["translator"]->translate("None (select)") . "</a> |\n";
            $content .= "<a href=\"javascript:invertBoxes(document.forms.$formname);\">" . $GLOBALS["translator"]->translate("Invert") . "</a> |\n";
            $content .= "<a href=\"javascript:rangeBoxes(document.forms.$formname);\">" . $GLOBALS["translator"]->translate("Range") . "</a>\n";
        }
        return $content;
    }// END FUNC _checkboxLinks


    /**
    * Reads $_REQUEST var and takes appropriate action
    *
    * @author   Don Seiler <don@NOSPAM.seiler.us>
    */
    function action()
    {
        $title = "ZUUL: ";
        $content = NULL;
        //if(ZUUL_Message::isMessage($this->message))
            //$this->message->display();

        if($this->alive()) {
            if(isset($_REQUEST["ZUUL_CORE_OP"])) {
                switch($_REQUEST["ZUUL_CORE_OP"]) {
                    // I'm starting with the basic list of "help" commands
                    // Plan to add "longhelp" commands eventually
                    case "vm":
                        $title .= $GLOBALS["translator"]->translate("View Connected Servers");
                        $content = $this->_vm();
                        break;
        
                    case "vma":
                        $title .= $GLOBALS["translator"]->translate("View All Servers");
                        $content = $this->_vm(TRUE);
                        break;
        
                    case "c":
                        $title .= $GLOBALS["translator"]->translate("Connect to Server");
                        $content = $this->_c();
                        break;
        
                    case "x":
                        $title .= $GLOBALS["translator"]->translate("Disconnect from Server");
                        $content = $this->_x();
                        break;
        
                    case "vd":
                        $title .= $GLOBALS["translator"]->translate("View Downloads");
                        $content = $this->_vd();
                        break;
        
                    case "cancel":
                        $title .= $GLOBALS["translator"]->translate("View Downloads");
                        $content = $this->_cancel();
                        break;
        
                    case "pause":
                        $title .= $GLOBALS["translator"]->translate("View Downloads");
                        $content = $this->_pause();
                        break;
        
                    case "resume":
                        $title .= $GLOBALS["translator"]->translate("View Downloads");
                        $content = $this->_resume();
                        break;
        
                    case "s":
                        $title .= $GLOBALS["translator"]->translate("Search");
                        $content = $this->_s();
                        break;
        
                    case "vr":
                        $title .= $GLOBALS["translator"]->translate("View Search Results");
                        $content = $this->_vr();
                        break;
    
                    case "forget":
                        $title .= $GLOBALS["translator"]->translate("Forget Search");
                        $content = $this->_forget();
                        break;
        
                    case "d":
                        $title .= $GLOBALS["translator"]->translate("Download Search Result");
                        $content = $this->_d();
                        break;
    
                    case "dllink":
                        $title .= $GLOBALS["translator"]->translate("Download ED2K Links");
                        $content = $this->_dllink();
                        break;
    
                    case "dllinks":
                        $title .= $GLOBALS["translator"]->translate("Download ED2K Links");
                        $content = $this->_dllinks();
                        break;
    
                    case "force_download":
                        $title .= $GLOBALS["translator"]->translate("Force Download");
                        $content = $this->_force_download();
                        break;
    
                    case "uploaders":
                        $title .= $GLOBALS["translator"]->translate("View Uploads");
                        $content = $this->_uploaders();
                        break;
        
                    case "vo":
                        $title .= $GLOBALS["translator"]->translate("Options");
                        $content = $this->_vo();
                        break;
    
                    case "voo":
                        $title .= $GLOBALS["translator"]->translate("Options");
                        $content = $this->_vo(TRUE);
                        break;
    
                    case "save":
                        $title .= $GLOBALS["translator"]->translate("Save MLdonkey Options");
                        $content = $this->_save();
                        break;
        
                    case "kill":
                        $title .= $GLOBALS["translator"]->translate("Kill MLdonkey");
                        $content = $this->_kill();
                        break;
    
    
                    case "priority":
                        $title .= $GLOBALS["translator"]->translate("Set Download Priority");
                        $content = $this->_priority();
                        break;
    
                    case "bulk":
                        $title .= $GLOBALS["translator"]->translate("Bulk File Operations");
                        $content = $this->_bulk();
                        break;
        
                    case "commandline":
                        $title .= $GLOBALS["translator"]->translate("MLdonkey Command Line");
                        $content = $this->_commandline();
                        break;
    
                    case "commit":
                        $title .= $GLOBALS["translator"]->translate("Commit Completed Downloads");
                        $content = $this->_commit();
                        break;
        
                    case "rename":
                        $title .= $GLOBALS["translator"]->translate("View Downloads");
                        $content = $this->_rename();
                        break;
        
                    default:
                        $title .= $GLOBALS["translator"]->translate("View Downloads");
                        $content = $this->_vd();
                }
            } else {
                $title .= $GLOBALS["translator"]->translate("View Downloads");
                $content = $this->_vd();
            }
        } else {
            if(isset($_REQUEST["ZUUL_CORE_OP"])) {
                switch($_REQUEST["ZUUL_CORE_OP"]) {
                    // I'm starting with the basic list of "help" commands
                    // Plan to add "longhelp" commands eventually
                    case "start":
                        $title .= $GLOBALS["translator"]->translate("Starting MLdonkey");
                        $content = $this->_start();
                        break;

                    default:
                        $title .= $GLOBALS["translator"]->translate("Welcome");
                        $content = "";
                }
            } else {
                $title .= $GLOBALS["translator"]->translate("Welcome");
                $content = "";
            }
        }
    
        return displayHeader($title) . $content . displayFooter();

    }// END FUNC action

}// END CLASS ZUUL_Core

?>
