<?php
/* vim: set expandtab tabstop=4 shiftwidth=4: */

/**
* File class
* 
* Represents an edonkey file (upload or download)
*
* @version  $Id: File.php,v 1.22 2003/08/14 13:03:31 cyb0rg Exp $
* @author   Don Seiler <don@NOSPAM.seiler.us>
* @package  zuul
*/
class ZUUL_File {
    
    /**
    * MD4 Hash
    *
    * @var      string
    * @access   public
    */
    var $md4hash = NULL;


    /**
    * Network (eDonkey, Direct Connect, Open Napster, BitTorrent, etc.)
    *
    * @var      string
    * @access   public
    */
    var $network = NULL;


    /**
    * mldonkey Number
    *
    * @var      string
    * @access   public
    */
    var $nr = NULL;


    /**
    * Priority
    *
    * @var      string
    * @access   public
    */
    var $priority = NULL;


    /**
    * Chunks
    *
    * @var      string
    * @access   public
    */
    var $chunks = NULL;


    /**
    * Transfer rate
    *
    * @var      string
    * @access   public
    */
    var $speed = NULL;


    /**
    * Size of file
    *
    * @var      int
    * @access   public
    */
    var $size = NULL;


    /**
    * Amount uploaded
    *
    * @var      int
    * @access   public
    */
    var $uploaded = NULL;


    /**
    * Amount downloaded
    *
    * @var      int
    * @access   public
    */
    var $downloaded = NULL;


    /**
    * Percent downloaded
    *
    * @var      double
    * @access   public
    */
    var $percent = NULL;


    /**
    * Estimated time of completion at current $speed
    * 
    * @var      string
    * @access   public
    */
    var $eta = NULL;

    /**
    * Name of file
    *
    * @var      string
    * @access   public
    */
    var $name = NULL;


    /**
    * Upload user
    *
    * @var      string
    * @access   public
    */
    var $user = NULL;


    /**
    * Upload user's client
    *
    * @var      string
    * @access   public
    */
    var $client = NULL;


    /**
    * Upload user's score
    *
    * @var      string
    * @access   public
    */
    var $uploadscore = NULL;


    /**
    * Days in download list
    *
    * @var      string
    * @access   public
    */
    var $daysinlist = NULL;


    /**
    * Days since last seen
    *
    * @var      string
    * @access   public
    */
    var $lastseen = NULL;


    /**
    * Whether or not file is finished downloading
    *
    * @var       boolean
    * @access   public
    */
    var $finished = FALSE;


    /**
    * File aging information
    * (conglomeration of daysinlist and lastseen)
    *
    * @var      string
    * @access   public
    */
    var $old = NULL;


    /**
    * Array of sources for this file
    *
    * @var      string
    * @access   public
    */
    var $sources = NULL;


    /**
    * Array of different names for this file
    *
    * @var      array
    * @access   public
    */
    var $names;




    function ZUUL_File($line = NULL, $up = FALSE, $finished = FALSE)
    {
        if(isset($line)) {
            // $line is an mldonkey response line
            // parse code lifted from phpeselgui
            // http://phpeselgui.sf.net
            if(ereg("^\[", $line) && !ereg("^\[ Num", $line)) {
                $line = preg_replace("/\[/", "", $line);
                $line = preg_replace("/\Direct Connect /", "DC ", $line);
                $line = preg_replace("/\Open Napster /", "OpenNap ", $line);
                $line = preg_replace("/\W*\]/", "", $line);
                $line = preg_replace("/\s+/", " ", $line);
                if(!$up) {
                    $this->finished = $finished;
                    if($this->finished) {
                        /* delete all spaces at the beginning of $line */
                        $line=preg_replace("/^[ ]*\[/","[",$line);
                        $temp=explode(" ", $line);
                        $length=sizeof($temp);

                        $this->network = $temp[0];
                        $this->nr = $temp[1];
                        $this->size = $temp[$length-3];
                        $this->md4hash = $temp[$length-2];
    
                        $text = "";
                        $ni = 2;
                        while($ni < $length-3) {
                            $text .= $temp[$ni] . " ";
                            $ni++;
                        }
                        $this->name = $text;

                    } else {
                        $temp=explode(" ", $line);
                        $length=sizeof($temp);
                        // Parse Downloads
                        $this->priority = $temp[$length-2];
                        $this->network = $temp[0];
                        $this->nr = $temp[1];
                        $this->speed = $temp[$length-3];
                        if($this->speed == "Paused")
                            $this->speed = -1;
                        $old = $temp[$length-4]; 
                        list($this->daysinlist,$this->lastseen) = explode(":", $old);
                        $this->old = $old;
                        $this->size = $temp[$length-5]; 
                        $this->downloaded = $temp[$length-6]; 
                        $this->percent = $temp[$length-7]; 
        
                        if($this->speed <= 0)
                            $this->eta = -1;
                        else
                            $this->eta = ($this->size - $this->downloaded) / (1024 * $this->speed);
        
                        $text = "";
                        $ni = 2;
                        while($ni < $length-7) {
                            $text .= $temp[$ni] . " ";
                            $ni++;
                        }
                        $this->name = $text;
                    }
                } else {
                    $temp=explode(" ", $line);
                    $length=sizeof($temp);
                    // Parse Uploads
                    /*
                    $c = 0;
                    foreach($temp as $t) {
                        echo "$c = $t<br />";
                        $c++;
                    }
                    */
                    /*
                    Sometimes emule concats 4 and 5
                    0 = Donkey
                    1 = 267
                    2 = 18
                    3 = 80.135.50.180:4662
                    4 = WGfreaksMLdonkey
                    5 = client:
                    6 = oML
                    7 = downloaded:
                    8 = 0
                    9 = uploaded:
                    10 = 675840
                    11 = 
                    */
                    $this->network = $temp[0];
                    $this->nr = $temp[1];
                    $this->uploadscore = $temp[2];
                    $this->ip = $temp[3];
                    $this->user = $temp[4];
                    if(preg_match("/client:/",$this->user)) {
                        $this->client = $temp[5];
                        $this->downloaded = $temp[7];
                        $this->uploaded = $temp[9];
                    } else {
                        $this->client = $temp[6];
                        $this->downloaded = $temp[8];
                        $this->uploaded = $temp[10];
                    }
                }
            } else {
                $this = NULL;
            }
        }
    }// END FUNC ZUUL_File


    function getDetails($lines = NULL)
    {
        if(!isset($this->nr))
            exit($GLOBALS["translator"]->translate("NR for file not set.  Cannot view download details."));

        $c = NULL;
        if (is_null($lines)) {
            $mldarray = $GLOBALS["core"]->mldonkey("vd " . $this->nr);
        } else {
            $mldarray = $lines;
        }
        for($i=0; $i<sizeof($mldarray); $i++) {
            if(preg_match("/^\[/", $mldarray[$i])) {
                $c = $i;
                break;
            }
        }

        // First line has network, nr, md4hash, size and downloaded
        $line = $mldarray[$c];
        $line = preg_replace("/\[/", "", $line);
        $line = preg_replace("/\Direct Connect /", "DC ", $line);
        $line = preg_replace("/\Open Napster /", "OpenNap ", $line);
        $line = preg_replace("/\W*\]/", "", $line);
        $line = preg_replace("/\s+/", " ", $line);
        $temp = explode(" ", $line);
        $length=sizeof($temp);
        $this->network      = $temp[0];
        $this->md4hash      = $temp[2];
        $this->size         = $temp[3];
        $this->downloaded   = $temp[4];
        $c++;

        // Second line has priority
        $line = $mldarray[$c];
        $temp = explode(" ", $line);
        $this->priority     = $temp[1];
        $c++;

        // Third line has chunks
        $line = $mldarray[$c];
        $line = preg_replace("/\[/", "", $line);
        $line = preg_replace("/\W*\]/", "", $line);
        $line = preg_replace("/\s+/", " ", $line);
        $temp=explode(" ", $line);
        $this->chunks       = $temp[1];
        $c++;

        // Fourth line repeats md4 hash, skip
        $c++;

        // Next set of lines until "<int> sources:" are names in parens
        for($i=$c; $i<sizeof($mldarray); $i++) {
            $line = $mldarray[$i];
            if(preg_match("/sources:/", $line)) {
                //$temp = explode(" ", $line);
                //$this->numsources = $temp[0];
                $c++;
                break;
            } else {
                // Strip off surrounding parens
                $line = preg_replace("/^\(/", "", $line);
                $line = preg_replace("/\W*\)$/", "", $line);
                $this->names[] = $line;
            }
            $c++;
        }

        // Rest of the lines until blank line are sources
        for($i=$c; $i<sizeof($mldarray); $i++) {
            $line = $mldarray[$i];
            if(preg_match("/^$/", trim($line)))
                break;

            $this->sources[] = $line;
        }

        // If $this->name isn't set, set it with first name entry
        if(!isset($this->name)) {
            $this->name = $this->names[0];
        }

    }// END FUNC getDetails


    function getED2KLink()
    {
        return $this->ed2kLink($this->name, $this->size, $this->md4hash);
    }// END FUNC getED2KLink


    function ed2kLink($name, $size, $md4hash)
    {
        if($this->network == "Donkey")
            return "ed2k://|file|" . trim($name) . "|$size|$md4hash|/";
        else
            return NULL;
    }


    function hrETA()
    {
        if(($this->eta > 365*24*60*60) || ($this->eta < 0))
            return "unknown";

        $result = NULL;

        $secs = $this->eta;

        $spf = array("%02d", "%02d:", "%2d:", "%dd ");
        $stf = array(60, 60, 24, 365);

        for ($i = 0; $secs > 0; $i++) {
            $tmp = $secs % $stf[$i];
            $result = sprintf($spf[$i],$tmp) . $result;
            $secs = floor($secs / $stf[$i]);
        }

        return $result;
    }// END FUNC hrETA


    function hrDownloaded()
    {
        return $this->hrSizes($this->downloaded);
    }// END FUNC hrDownloaded


    function hrUploaded()
    {
        return $this->hrSizes($this->uploaded);
    }// END FUNC hrDownloaded


    function hrSize()
    {
        return $this->hrSizes($this->size);
    }// END FUNC hrSize


    function hrSizes($size)
    {
        $stf = array(" Bytes", " KB", " MB", " GB", "TB");
        for ($i = 0; $size > 1024; $i++)
            $size = $size/1024;

        return round($size,2) . $stf[$i];
    }// END FUNC hrSizes


    function setFilename($filename)
    {
        $this->name = $filename;
    }// END FUNC setFilename


    function hrPriority()
    {
        $priorities = $this->getPriorities();
        if(array_key_exists($this->priority, $priorities))
            return $priorities[$this->priority];
        else
            return $this->priority;
    }// END FUNC hrPriority


    function getPriorities()
    {
        return array(
                        "0"=>$GLOBALS["translator"]->translate("None (priority)"),
                        "25"=>$GLOBALS["translator"]->translate("Low"),
                        "50"=>$GLOBALS["translator"]->translate("Medium"),
                        "75"=>$GLOBALS["translator"]->translate("High")
                    );
 
    }// END FUNC getPriorities

    
    function isPaused()
    {
        return ($this->speed == "Paused" or $this->speed == -1);
    }// END FUNC isPaused

}// END CLASS ZUUL_File
