<?php
/* vim: set expandtab tabstop=4 shiftwidth=4: */

/**
* Search Result class
* 
* Represents an edonkey search result
*
* @version  $Id: SearchResult.php,v 1.5 2003/06/24 14:21:57 cyb0rg Exp $
* @author   Don Seiler <don@NOSPAM.seiler.us>
* @package  zuul
*/
class ZUUL_SearchResult {
    
    /**
    * Search result number
    *
    * @var      string
    * @access   public
    */
    var $resultnum = NULL;


    /**
    * Network
    *
    * @var      string
    * @access   public
    */
    var $network = NULL;


    /**
    * File name
    *
    * @var      string
    * @access   public
    */
    var $name = NULL;


    /**
    * File size
    *
    * @var      string
    * @access   public
    */
    var $size = NULL;


    /**
    * File Hash Value
    *
    * @var      string
    * @access   public
    */
    var $md4hash = NULL;


    /**
    * File availability
    *
    * @var      string
    * @access   public
    */
    var $availability = NULL;


    function ZUUL_SearchResult($linearray = NULL)
    {
        if(isset($linearray) and (count($linearray) > 0)) {
            $gotfirstline = 0;
            foreach($linearray as $line) {
                // $line is an mldonkey response line
                // basic parse code lifted from phpeselgui
                // http://phpeselgui.sf.net
                $found=array();
                if (!$gotfirstline) {
                    $line = preg_replace("/] Direct Connect /", "] DC ", $line);
                    $line = preg_replace("/] Open Napster /", "] OpenNap ", $line);
                    $gotfirstline = 1;
                }
                if (preg_match("/^\[\ *(\d*)\]\ (\w+)\ (.*)/", $line, $found)) {
                    $this->resultnum  = trim($found[1]);
                    $this->network    = trim($found[2]);
                    $this->name       = trim($found[3]);
                } elseif (preg_match("/^(\d+) (\w+) (\d+)/", $line, $found)) {
                    $this->size          = trim($found[1]);
                    $this->md4hash       = trim($found[2]);
                    $this->availability  = trim($found[3]);
                } elseif (preg_match("/^(.*)/", $line, $found)) {
                    // At this point $found[1] is an alternate name
                    // for the file.  For now we won't do anything
                    // with it
                } else {
                    // Note that we'll basically never get here...
                    $this = NULL;
                }
            }
        }
    }// END FUNC ZUUL_SearchResult

    /**
    * Parser for "vr" command
    *
    * Takes in an array of the complete mldonkey output and
    * returns an array of ZUUL_SearchResult objects.
    *
    * $result is used internally to store the array of lines
    *       which define a result.  This is at least two lines
    *       because of the way mldonkey ouputs search results,
    *       and may be however long mldonkey deems fit.
    * $started is used internally to determine if the result
    *       list has been started yet, to skip forward past
    *       the mldonkey headers
    * $process is used internally to try processing an
    *       individual result.  When this function sees
    *       another search result header or a blank line,
    *       it will instantiate a ZUUL_SearchResult object
    *       with the lines held in $result
    *
    * @author   CJ Kucera <zuul@NOSPAM.apocalyptech.com>
    * @access   public
    */
    function getResults($linearray)
    {
        $return_results = array();
        $result = array();
        $started = 0;
        for($i = 0; $i < count($linearray); $i++) {
            $line = $linearray[$i];
            $process = 0;

            if (preg_match("/^\[\ *(\d+)\]/", $line)) {
                $started = 1;
                $process = 1;
            } elseif (preg_match("/^\s*$/", $line)) {
                $process = 1;
            }

            if ($process) {
                if (count($result) > 0) {
                    $return_results[] = new ZUUL_SearchResult($result);
                }
                $result = array();
            }

            if ($started) {
                $result[] = $line;
            }

        }
        return $return_results;
    }// END FUNC getResults

}// END CLASS ZUUL_SearchResult
