#!/usr/bin/perl -w
# vim: set expandtab tabstop=4 shiftwidth=4:

# This script updates an AOL language file to include all phrases
# from the "master" file.  When run with no arguments, it will
# simply update lang/aol.txt from lang/english.txt
#
# If you pass in a language (ie: dutch) as the first argument, it
# will update lang/aol-dutch.txt from lang/dutch.txt
#
# The program will only add new entries - it will not change existing
# ones or delete lines that shouldn't exist anymore.  Use translint.sh
# to check for lines that shouldn't exist.
#
# The translation code was taken basically from
# http://ssshotaru.homestead.com/files/aolertranslator.html
# Despite the fact that this code bears just about NO resemblance to the
# Javascript at that page, I basically constructed this line-for-line from
# that source.  This one doubtless behaves differently on account of how
# I decided to do things, but I'm happy enough with it right now.

use strict;
use Cwd 'abs_path';

# Perform a random substitution
sub s
{
    return $_[rand @_];
}

# Handle punctuation
sub punc
{
    my $mark = shift;
    my $output = "";
    my @ques = qw(! ?);
    my @other = qw(! 1);
    if ($mark eq "?")
    {
        $output .= $mark;
        for (my $i=0; $i< ((rand()*100)%5)+4; $i++)
        {
            $output .= $ques[rand @ques];
        }
    }
    else
    {
        $output .= "!";
        for (my $i=0; $i< ((rand()*100)%5)+2; $i++)
        {
            $output .= $other[rand @other];
        }
        $output .= "?";
        for (my $i=0; $i< ((rand()*100)%5)+3; $i++)
        {
            $output .= $other[rand @other];
        }
    }
    if ((rand()*100)%2) { $output .= " OMG"; }
    if ((rand()*100)%2) { $output .= " WTF"; }
    if ((rand()*100)%2) { $output .= " LOL"; }
    return $output;
}

# The main translation subroutine
sub translateText()
{
	my $text = shift;
    my @ary;
    my $output = "";

    # uppercase, of course
    $text = uc $text;

    # phrase substitutions
    $text =~ s/WHAT THE FUCK/WTF/g;
    $text =~ s/WHAT THE HELL/WTH/g;
    $text =~ s/WHAT THE/WT/g;
    $text =~ s/BE RIGHT BACK/BRB/g;
    $text =~ s/OH MY GOD/OMG/g;
    $text =~ s/I AM/IMA/g;
    $text =~ s/SEE YOU/&s(qw(CU CYA))/eg;
    $text =~ s/SEE YA/&s(qw(CU CYA))/eg;

    @ary = split /\s/, $text;
    foreach my $word (@ary)
    {
        # word substitutions
        $word =~ s/YOUR/UR/g;
        $word =~ s/YOU'RE/UR/g;
        $word =~ s/YOU/U/g;
        $word =~ s/WHAT/WUT/g;
        $word =~ s/ARE/R/g;
        $word =~ s/WHY/Y/g;
        $word =~ s/BECAUSE/B\/C/g;
        $word =~ s/OH/O/g;
        $word =~ s/MY/MAH/g;
        $word =~ s/NEW/NU/g;
        $word =~ s/WITH/WIT/g;
        $word =~ s/REALLY/RILLY/g;
        $word =~ s/PLEASE/PLZ/g;
        $word =~ s/THANKS/THX/g;
        $word =~ s/OK/K/g;
        $word =~ s/OKAY/K/g;
        $word =~ s/LIBRARY/LIBERRY/g;
        $word =~ s/BE/B/g;
        $word =~ s/CK/C/g;
        $word =~ s/I'M/IM/g;
        $word =~ s/ATER/8R/g;
        $word =~ s/SCHOOL/SKOOL/g;
        $word =~ s/OO/UU/g;
        $word =~ s/OU/UU/g;
        $word =~ s/TO/2/g;
        $word =~ s/TOO/2/g;
        $word =~ s/THAT/TAHT/g;

        # More complex things
        $word =~ s/ING$/NG/g;
        $word =~ s/IE/EI/g;
        $word =~ s/I(.)E/IE$1/g;

        # Random things
        $word =~ s/THE/&s(qw(TEH DA))/eg;
        $word =~ s/E/&s(qw(E A 3))/eg;

        # Circular
        if ($word =~ /THERE/)
        {
            $word =~ s/THERE/&s(qw(THEIR THEYRE))/eg;
        }
        elsif ($word =~ /THEIR/)
        {
            $word =~ s/THEIR/&s(qw(THERE THEYRE))/eg;
        }
        elsif ($word =~ /THEY'RE/)
        {
            $word =~ s/THEY'RE/&s(qw(THERE THEIR))/eg;
        }

        # Delete double letters
        $word =~ s/(.)\1/$1/g;

        # Get rid of non-alphanumerics (this removes punctuation too!)
        #$word =~ s/\W//g;

        $output .= "$word ";
    }
    chop $output;

    # Punctuation
    $output =~ s/([\.\?\!])/&punc($1)/ge;

    return $output;
}

###
### And now, the actual processing
###

# Grab Zuul's home dir
my $zuul_home = abs_path($0);
$zuul_home =~ s/\/misc\/aol\.pl//;

# Find out what language we're using
my $lang = "english";
my $ext = ".txt";
if ($ARGV[0] and $ARGV[0] ne "")
{
    $lang = $ARGV[0];
    $ext = "-$lang.txt";
}
if ($lang =~ /^aol/)
{
    print "Don't run this program on an AOL-language.\n";
    exit;
}

# Grab our source language
my %translations = ();
my $DF;
if (-f "$zuul_home/lang/$lang.txt")
{
    open DF, "$zuul_home/lang/$lang.txt";
    while (<DF>)
    {
        chomp;
        my ($orig, $delim, $new) = split ":";
        $translations{$orig} = $new;
    }
    close DF;
}
else
{
    print "Language $lang does not exist!\n";
    exit;
}

# Grab our current AOL language
my %aoltranslations = ();
if (-f "$zuul_home/lang/aol$ext")
{
    open DF, "$zuul_home/lang/aol$ext";
    while (<DF>)
    {
        chomp;
        my ($orig, $delim, $new) = split ":";
        $aoltranslations{$orig} = $new;
    }
    close DF;
}

# Weed out ones that already exist
foreach my $aolkey (keys %aoltranslations)
{
    delete $translations{$aolkey};
}

# And now add in the new translations
foreach my $newkey (keys %translations)
{
    $aoltranslations{$newkey} = &translateText($translations{$newkey});
    print "Added '" . $newkey . "'\n";
}

# Write out the new file
open DF, ">$zuul_home/lang/aol$ext";
foreach my $aolkey (sort {uc($a) cmp uc($b)} keys %aoltranslations)
{
    print DF $aolkey . ":|:" . $aoltranslations{$aolkey} . "\n";
}
close DF;

# ... and we're done!
